import logging

import click

from pypgatk.cgenomes.cbioportal_downloader import CbioPortalDownloadService
from pypgatk.toolbox.exceptions import AppConfigException


@click.command('cbioportal-downloader', short_help=' Command to download the the cbioportal studies')
@click.option('--config_file', '-c', help='Configuration file for the ensembl data downloader pipeline',
              default='config/cbioportal_config.yaml')
@click.option('--output_directory', '-o', help='Output directory for the peptide databases',
              default="./database_cbioportal/")
@click.option('--list_studies', '-l',
              help='Print the list of all the studies in cBioPortal (https://www.cbioportal.org)', is_flag=True)
@click.option('--download_study', '-d',
              help="Download a specific Study from cBioPortal -- (all to download all studies)")
@click.option('--multithreading', '-th', help=' Enable multithreading to download multiple files ad the same time', is_flag=True)
@click.pass_context
def cbioportal_downloader(ctx, config_file, output_directory, list_studies, download_study, multithreading):
    if config_file is None:
        msg = "The config file for the pipeline is missing, please provide one "
        logging.error(msg)
        raise AppConfigException(msg)

    pipeline_arguments = {}
    if output_directory is not None:
        pipeline_arguments[CbioPortalDownloadService.CONFIG_OUTPUT_DIRECTORY] = output_directory
    if list_studies:
        pipeline_arguments[CbioPortalDownloadService.CONFIG_LIST_STUDIES] = list_studies
    if multithreading is not None:
        pipeline_arguments[CbioPortalDownloadService.CONFIG_MULTITHREADING] = multithreading

    cbioportal_downloader_service = CbioPortalDownloadService(config_file, pipeline_arguments)

    if list_studies:
        list_of_studies = cbioportal_downloader_service.get_cancer_studies()
        print(list_of_studies)

    if download_study is not None:
        cbioportal_downloader_service.download_study(download_study)


import re

from Bio import SeqIO

from pypgatk.cgenomes.models import SNP
from pypgatk.toolbox.general import ParameterConfiguration


class CancerGenomesService(ParameterConfiguration):
    CONFIG_CANCER_GENOMES_MUTATION_FILE = 'mutation_file'
    CONFIG_COMPLETE_GENES_FILE = "genes_file"
    CONFIG_OUTPUT_FILE = "output_file"
    CONFIG_COSMIC_DATA = "cosmic_data"
    CONFIG_KEY_DATA = 'proteindb'
    CONFIG_TISSUE_INFO = 'tissue_info'
    TISSUE_TYPE = "tissue_type"
    SPLIT_BY_TISSUE_TYPE = "split_by_tissue_type"
    CLINICAL_SAMPLE_FILE = 'clinical_sample_file'
    
    def __init__(self, config_file, pipeline_arguments):
        """
        Init the class with the specific parameters.
        :param config_file configuration file
        :param pipeline_arguments pipelines arguments
        """
        super(CancerGenomesService, self).__init__(self.CONFIG_COSMIC_DATA, config_file, pipeline_arguments)

        if self.CONFIG_CANCER_GENOMES_MUTATION_FILE in self.get_pipeline_parameters():
            self._local_mutation_file = self.get_pipeline_parameters()[self.CONFIG_CANCER_GENOMES_MUTATION_FILE]

        if self.CONFIG_COMPLETE_GENES_FILE in self.get_pipeline_parameters():
            self._local_complete_genes = self.get_pipeline_parameters()[self.CONFIG_COMPLETE_GENES_FILE]

        if self.CONFIG_OUTPUT_FILE in self.get_pipeline_parameters():
            self._local_output_file = self.get_pipeline_parameters()[self.CONFIG_OUTPUT_FILE]
        
        self._tissue_type = self.get_multiple_options(
            self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_TISSUE_INFO][self.TISSUE_TYPE])
        if self.TISSUE_TYPE in self.get_pipeline_parameters():
            self._tissue_type = self.get_multiple_options(self.get_pipeline_parameters()[self.TISSUE_TYPE])

        self._split_by_tissue_type = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_TISSUE_INFO][self.SPLIT_BY_TISSUE_TYPE]
        if self.SPLIT_BY_TISSUE_TYPE in self.get_pipeline_parameters():
            self._split_by_tissue_type = self.get_pipeline_parameters()[self.SPLIT_BY_TISSUE_TYPE]
        
        self._local_clinical_sample_file = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_TISSUE_INFO][self.CLINICAL_SAMPLE_FILE]
        if self.CLINICAL_SAMPLE_FILE in self.get_pipeline_parameters():
            self._local_clinical_sample_file = self.get_pipeline_parameters()[self.CLINICAL_SAMPLE_FILE]
    
    @staticmethod
    def get_multiple_options(options_str: str):
        """
        This method takes an String like option1, option2, ... and produce and array [option1, option2,... ]
        :param options_str:
        :return: Array
        """
        return list(map(lambda x: x.strip(), options_str.split(",")))

    def cosmic_to_proteindb(self):
        """
        This function translate the mutation file + COSMIC genes into a protein Fasta database. The
        method write into the file system the output Fasta.
        :return:
        """
        self.get_logger().debug("Starting reading the All cosmic genes")
        COSMIC_CDS_DB = SeqIO.index(self._local_complete_genes, 'fasta')  # All_COSMIC_Genes.fasta

        cosmic_input = open(self._local_mutation_file, 'r')  # CosmicMutantExport.tsv

        header = cosmic_input.readline().split("\t")
        regex = re.compile('[^a-zA-Z]')
        gene_col = header.index("Gene name")
        enst_col = header.index("Accession Number")
        cds_col = header.index("Mutation CDS")
        aa_col = header.index("Mutation AA")
        muttype_col = header.index("Mutation Description")
        tissue_col = header.index('Primary site')
        
        output = open(self._local_output_file, 'w')

        mutation_dic = {}
        tissue_mutations_dict = {}
        nucleotide = ["A", "T", "C", "G"]
        self.get_logger().debug("Reading input CosmicMutantExport.tsv ...")
        line_counter = 1
        for line in cosmic_input:
            if line_counter % 10000 == 0:
                msg = "Number of lines finished -- '{}'".format(line_counter)
                self.get_logger().debug(msg)
            line_counter += 1
            row = line.strip().split("\t")
            #filter out mutations from unspecified tissues
            if row[tissue_col] not in self._tissue_type and self._tissue_type!=['all']:
                continue
            
            if "coding silent" in row[muttype_col]:
                continue

            snp = SNP(gene=row[gene_col], mrna=row[enst_col], dna_mut=row[cds_col], aa_mut=row[aa_col],
                      type=row[muttype_col])
            header = "COSMIC:%s:%s:%s" % (snp.gene, snp.aa_mut, snp.type.replace(" ", ""))
            try:
                seq = COSMIC_CDS_DB[snp.gene].seq
            except KeyError:  # geneID is not in All_COSMIC_Genes.fasta
                continue

            mut_pro_seq = ""
            if "?" not in row[cds_col]:  # unambiguous DNA change known in CDS sequence
                positions = re.findall(r'\d+', snp.dna_mut)
                if ">" in snp.dna_mut and len(positions) == 1:  # Substitution
                    tmplist = snp.dna_mut.split(">")
                    ref_dna = re.sub("[^A-Z]+", "", tmplist[0])
                    mut_dna = re.sub("[^A-Z]+", "", tmplist[1])
                    index = int(positions[0]) - 1

                    if ref_dna == str(seq[index]).upper() and mut_dna in nucleotide:  #
                        seq_mut = seq[:index] + mut_dna + seq[index + 1:]
                        mut_pro_seq = seq_mut.translate(to_stop=True)
                elif "ins" in snp.dna_mut:
                    index = snp.dna_mut.index("ins")
                    insert_dna = snp.dna_mut[index + 3:]
                    if insert_dna.isalpha():
                        #print(insert_dna, snp.dna_mut, snp.aa_mut)
                        ins_index1 = int(positions[0])
                        seq_mut = seq[:ins_index1] + insert_dna + seq[ins_index1:]
                        mut_pro_seq = seq_mut.translate(to_stop=True)

                elif "del" in snp.dna_mut:
                    if len(positions) == 2:
                        del_index1 = int(positions[0]) - 1
                        del_index2 = int(positions[1])
                        seq_mut = seq[:del_index1] + seq[del_index2:]
                        mut_pro_seq = seq_mut.translate(to_stop=True)
                    elif len(positions) == 1:
                        del_index1 = int(positions[0]) - 1
                        seq_mut = seq[:del_index1] + seq[del_index1 + 1:]
                        mut_pro_seq = seq_mut.translate(to_stop=True)
            else:
                if "?" not in row[aa_col]:  # unambiguous aa change known in protein sequence
                    positions = re.findall(r'\d+', snp.aa_mut)
                    protein_seq = str(seq.translate(to_stop=True))

                    if "Missense" in snp.type:
                        mut_aa = snp.aa_mut[-1]
                        index = int(positions[0]) - 1
                        mut_pro_seq = protein_seq[:index] + mut_aa + protein_seq[index + 1:]
                    elif "Nonsense" in snp.type:
                        index = int(positions[0]) - 1
                        mut_pro_seq = protein_seq[:index]
                    elif "Insertion - In frame" in snp.type:
                        index = snp.aa_mut.index("ins")
                        insert_aa = snp.aa_mut[index + 3:]
                        if insert_aa.isalpha():
                            ins_index1 = int(positions[0])
                            mut_pro_seq = protein_seq[:ins_index1] + insert_aa + protein_seq[ins_index1:]
                    elif "Deletion - In frame" in snp.type:
                        if len(positions) == 2:
                            del_index1 = int(positions[0]) - 1
                            del_index2 = int(positions[1])
                            mut_pro_seq = protein_seq[:del_index1] + protein_seq[del_index2:]
                        elif len(positions) == 1:
                            del_index1 = int(positions[0]) - 1
                            mut_pro_seq = protein_seq[:del_index1] + protein_seq[del_index1 + 1:]
                    elif "Complex" in snp.type and "frameshift" not in snp.type:
                        try:
                            index = snp.aa_mut.index(">")
                        except ValueError:
                            # print (snp.gene,snp.mRNA,snp.dna_mut,snp.aa_mut,snp.type)
                            continue
                        mut_aa = snp.aa_mut[index + 1:]
                        if "deletion" in snp.type:
                            try:
                                del_index1 = int(positions[0]) - 1
                                del_index2 = int(positions[1])
                                mut_pro_seq = protein_seq[:del_index1] + mut_aa + protein_seq[del_index2:]
                            except IndexError:
                                # print (snp.gene,snp.mRNA,snp.dna_mut,snp.aa_mut,snp.type)
                                continue
                        elif "insertion" in snp.type:
                            try:
                                ins_index1 = int(positions[0]) - 1
                            except IndexError:
                                # print (snp.gene,snp.mRNA,snp.dna_mut,snp.aa_mut,snp.type)
                                continue
                            mut_pro_seq = protein_seq[:ins_index1] + mut_aa + protein_seq[ins_index1 + 1:]
                        elif "compound substitution" in snp.type:
                            if "*" not in mut_aa:
                                try:
                                    del_index1 = int(positions[0]) - 1
                                    del_index2 = int(positions[1])
                                    mut_pro_seq = protein_seq[:del_index1] + mut_aa + protein_seq[del_index2:]
                                except IndexError:
                                    # print (snp.gene,snp.mRNA,snp.dna_mut,snp.aa_mut,snp.type)
                                    continue
                            else:
                                try:
                                    del_index1 = int(positions[0]) - 1
                                    mut_pro_seq = protein_seq[:del_index1] + mut_aa.replace("*", "")
                                except IndexError:
                                    # print (snp.gene,snp.mRNA,snp.dna_mut,snp.aa_mut,snp.type)
                                    continue
            if mut_pro_seq != "":
                entry = ">%s\n%s\n" % (header, mut_pro_seq)
                if header not in mutation_dic:
                    output.write(entry)
                    mutation_dic[header] = 1
                
                if self._split_by_tissue_type:
                    try:
                        tissue_mutations_dict[row[tissue_col]][header] = entry
                    except KeyError:
                        tissue_mutations_dict[row[tissue_col]] = {header: entry}
            
        for tissue_type in tissue_mutations_dict.keys():
            with open(self._local_output_file.replace('.fa', '')+ '_' + regex.sub('', tissue_type) +'.fa', 'w') as fn:
                for header in tissue_mutations_dict[tissue_type].keys():
                    fn.write(tissue_mutations_dict[tissue_type][header])
            
        self.get_logger().debug("COSMIC contains in total {} non redundant mutations".format(len(mutation_dic)))
        cosmic_input.close()
        output.close()
        
    @staticmethod
    def get_sample_headers(header_line):
        try:
            tissue_type_col = header_line.index('CANCER_TYPE')
        except ValueError:
            print('CANCER_TYPE was not found in the header row:', header_line)
            return None, None
        try:
            sample_id_col = header_line.index('SAMPLE_ID')
        except ValueError:
            print('SAMPLE_ID was not found in the header row:', header_line)
            return None, None
        return tissue_type_col, sample_id_col
    
    def get_tissue_type_per_sample(self, local_clinical_sample_file):
        sample_tissue_type = {}
        if local_clinical_sample_file:
            with open(local_clinical_sample_file, 'r') as clin_fn:
                tissue_type_col, sample_id_col = None, None
                for line in clin_fn.readlines():
                    if line.startswith('#'):
                        continue
                    sl = line.strip().split('\t')
                    #check for header and re-assign columns
                    if 'SAMPLE_ID' in sl and 'CANCER_TYPE' in sl:
                        tissue_type_col, sample_id_col = self.get_sample_headers(sl)
                    if tissue_type_col and sample_id_col:
                        sample_tissue_type[sl[sample_id_col]] = sl[tissue_type_col].strip().replace(' ','_')
        return sample_tissue_type
    
    @staticmethod
    def get_mut_header_cols(header_cols, row, tissue_type, split_by_tissue_type):
        for col in header_cols.keys():
            header_cols[col] = row.index(col)
        
        #check if tissue type should be considered
        if tissue_type!=['all'] or split_by_tissue_type:
            try:
                header_cols["Tumor_Sample_Barcode"] = row.index("Tumor_Sample_Barcode")
            except ValueError:
                print("Tumor_Sample_Barcode was not found in the header {} of mutations file".format(row))
                header_cols["Tumor_Sample_Barcode"] = None
        
        return header_cols
    
    def cbioportal_to_proteindb(self):
        """cBioportal studies have a data_clinical_sample.txt file that shows the Primary Tumor Site per Sample Identifie
        it matches the the Tumor_Sample_Barcode column in the mutations file.
        """
        regex = re.compile('[^a-zA-Z]')
        mutfile = open(self._local_mutation_file, "r")
        fafile = SeqIO.parse(self._local_complete_genes, "fasta")
        output = open(self._local_output_file, "w")
        sample_tissue_type_dict = {}
        tissue_mutations_dict = {}
        
        seq_dic = {}
        for record in fafile:
            newacc = record.id.split(".")[0]
            if newacc not in seq_dic:
                seq_dic[newacc] = record.seq
        
        header_cols = {"HGVSc": None, "Transcript_ID": None, "Variant_Classification": None, 
                       "Variant_Type": None, "HGVSp_Short": None}
        nucleotide = ["A", "T", "C", "G"]
        mutclass = ["Frame_Shift_Del", "Frame_Shift_Ins", "In_Frame_Del", "In_Frame_Ins", "Missense_Mutation",
                    "Nonsense_Mutation"]
        
        # check if sample id and clinical files are given, if not and tissue type is required then exit
        if self._tissue_type!=['all'] or self._split_by_tissue_type:
            if self._local_clinical_sample_file:
                sample_tissue_type_dict = self.get_tissue_type_per_sample(self._local_clinical_sample_file)
                if sample_tissue_type_dict=={}:
                    return
            else:
                print('No clinical sample file is given therefore no tissue type can be detected.')
                return
            
        for i,line in enumerate(mutfile):
            row = line.strip().split("\t")
            if row[0]=='#':
                print("skipping line ({}): {}".format(i, row))
                continue
            #check for header in the mutations file and get column indecis
            if set(header_cols.keys()).issubset(set(row)):
                header_cols = self.get_mut_header_cols(header_cols, row, self._tissue_type, self._split_by_tissue_type)
            #check if any is none in header_cols then continue
            if None in header_cols.values():
                print("Incorrect header column is given")
                continue
            #get tissue type and check it
            tissue_type = None
            if self._tissue_type!=['all'] or self._split_by_tissue_type:
                try:
                    tissue_type = sample_tissue_type_dict[row[header_cols["Tumor_Sample_Barcode"]]]
                except KeyError:
                    if self._tissue_type!=['all'] or self._split_by_tissue_type:
                        print("No clinical info was found for sample {}. Skipping (line {}): {}".format(row[header_cols["Tumor_Sample_Barcode"]], i, line))
                        continue
                except IndexError:
                    print("No sampleID was found in (line {}): {}".format(i, row))
            if tissue_type not in self._tissue_type and self._tissue_type != ['all']:
                continue
            
            gene = row[0]
            try:
                pos = row[header_cols["HGVSc"]]
                enst = row[header_cols["Transcript_ID"]]

                seq_mut = ""
                aa_mut = row[header_cols["HGVSp_Short"]]

                vartype = row[header_cols["Variant_Type"]]
                varclass = row[header_cols["Variant_Classification"]]
            except IndexError:
                print("Incorrect line (i):", row)
                continue
            if varclass not in mutclass:
                continue
            
            try:
                seq = seq_dic[enst]
            except KeyError:
                print("%s not found:" % enst)
                continue
            
            if ":" in pos:
                cdna_pos = pos.split(":")[1]
            else:
                cdna_pos = pos

            if vartype == "SNP":
                try:
                    enst_pos = int(re.findall(r'\d+', cdna_pos)[0])
                except IndexError:
                    print("Incorrect SNP format or record", i, pos, line)
                    continue
                idx = pos.index(">")
                ref_dna = pos[idx - 1]
                mut_dna = pos[idx + 1]

                if mut_dna not in nucleotide:
                    print(mut_dna, "is not a nucleotide base", pos)
                    continue
                try:
                    if ref_dna == seq[enst_pos - 1]:
                        seq_mut = seq[:enst_pos - 1] + mut_dna + seq[enst_pos:]
                    else:
                        print("incorrect substitution, unmatched nucleotide", pos, enst)
                except IndexError:
                    print("incorrect substitution, out of index", pos)
            elif vartype == "DEL":
                try:
                    enst_pos = int(re.findall(r'\d+', cdna_pos.split("_")[0])[0])
                except IndexError:
                    print("incorrect del format or record", i, pos, line)
                    continue
                del_dna = pos.split("del")[1]
                if del_dna == seq[enst_pos - 1:enst_pos - 1 + len(del_dna)]:
                    seq_mut = seq[:enst_pos - 1] + seq[enst_pos - 1 + len(del_dna):]
                else:
                    print("incorrect deletion, unmatched nucleotide", pos)

            elif vartype == "INS":
                try:
                    enst_pos = int(re.findall(r'\d+', cdna_pos.split("_")[0])[0])
                except IndexError:
                    print("incorrect ins/dup format or record", i, pos, line)
                    continue
                if "ins" in pos:
                    ins_dna = pos.split("ins")[1]
                elif "dup" in pos:
                    ins_dna = pos.split("dup")[1]
                    if len(ins_dna) > 1:
                        enst_pos = int(re.findall(r'\d+', cdna_pos.split("_")[1])[0])
                else:
                    print("unexpected insertion format")
                    continue;

                seq_mut = seq[:enst_pos] + ins_dna + seq[enst_pos:]

            if seq_mut == "":
                continue;

            mut_pro_seq = seq_mut.translate(to_stop=True)
            if len(mut_pro_seq) > 6:
                header = "cbiomut:%s:%s:%s:%s" % (enst, gene, aa_mut, varclass)
                output.write(">%s\n%s\n" % (header, mut_pro_seq))
                
                if self._split_by_tissue_type:
                    try:
                        tissue_mutations_dict[tissue_type][header] = mut_pro_seq
                    except KeyError:
                        tissue_mutations_dict[tissue_type] = {header: mut_pro_seq}
                    
        output.close()
        mutfile.close()
        fafile.close()
        
        for tissue_type in tissue_mutations_dict.keys():
            with open(self._local_output_file.replace('.fa', '')+ '_' + regex.sub('', tissue_type) +'.fa', 'w') as fn:
                for header in tissue_mutations_dict[tissue_type].keys():
                    fn.write(">{}\n{}\n".format(header, tissue_mutations_dict[tissue_type][header]))
         




"""
Toolbox related to REST services
"""

import requests


def make_rest_request(url):
    response = requests.get(url, headers={"Content-Type": "application/json"})
    if not response.ok:
        response.raise_for_status()
    return response.json()


if __name__ == '__main__':
    print("ERROR: This script is part of a pipeline collection and it is not meant to be run in stand alone mode")



"""
Application wide exceptions
"""


class AppException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class AppConfigException(AppException):
    def __init__(self, value):
        super(AppConfigException, self).__init__(value)


class ConfigManagerException(Exception):
    def __init__(self, value):
        Exception.__init__(self)
        self.value = value

    def __str__(self):
        return repr(self.value)


class ToolBoxException(AppException):
    def __init__(self, value):
        super(ToolBoxException, self).__init__(value)


if __name__ == '__main__':
    print("ERROR: This script is part of a pipeline collection and it is not meant to be run in stand alone mode")



"""
Ensembl module related exceptions
"""
from pypgatk.toolbox.exceptions import AppException


class EnsemblServiceException(AppException):
    def __init__(self, value):
        super(EnsemblServiceException, self).__init__(value)


if __name__ == '__main__':
    print("ERROR: This script is part of a pipeline collection and it is not meant to be run in stand alone mode")



import getopt
import re
import sys
from collections import OrderedDict

from Bio import SeqIO


def TRYPSIN(proseq, miss_cleavage):
    peptides = []
    peptide = ''
    for c, aa in enumerate(proseq):
        peptide += aa
        next_aa = ''
        try:
            next_aa = proseq[c + 1]
        except IndexError:
            pass

        if aa in ['K', 'R'] and next_aa != 'P':  # for trypsin peptides
            if len(peptide) > 0:
                peptides.append(peptide)
            peptide = ''
            continue

    if len(peptide) > 0:
        peptides.append(peptide)

    peptides_with_miss_cleavage = []
    for i in range(1, miss_cleavage + 1):
        for j, pep in enumerate(peptides):
            if j + i < len(peptides):
                peptide = ''.join([x for x in (peptides[j:j + i + 1])])
                peptides_with_miss_cleavage.append(peptide)

    peptides.extend(peptides_with_miss_cleavage)

    return peptides


min = 7  # default minimum peptide length
max = 40  # default maximum peptide length
header_prefix = "Mutation"
cleavageMiss = 0  # number of misscleavage,default no misscleavage

if len(sys.argv[1:]) <= 1:  ### Indicates that there are insufficient number of command-line arguments
    print("Warning! wrong command!")
    print(
        "Example: python3 digest_mutant_protein.py --input mutproteins.1.fa, mutproteins.2.fa --fa knownproteins.fa --prefix Mutation --output mutpeptides.fa --min_len 7 --max_len 40 --miss 0")
else:
    options, remainder = getopt.getopt(sys.argv[1:], '',
                                       ['input=', 'fa=', 'output=', 'prefix=', 'min_len', 'max_len', 'miss'])
    for opt, arg in options:
        if opt == '--input':
            input_file = arg
        elif opt == '--output':
            output_file = arg
        elif opt == '--fa':
            fa_file = arg
        elif opt == '--prefix':
            header_prefix = arg
        elif opt == '--min_len':
            min = int(arg)
        elif opt == '--max_len':
            max = int(arg)
        elif opt == '--miss':
            cleavageMiss = int(arg)
        else:
            print("Warning! Command-line argument: %s not recognized. Exiting..." % opt);
            sys.exit()

handle1 = SeqIO.parse(fa_file, 'fasta')  # canonical protein sequences
peptidome = {}

for record in handle1:
    aa_seq = record.seq
    peptide_list = TRYPSIN(str(aa_seq), cleavageMiss)
    for peptide in peptide_list:
        if len(peptide) in range(min, max + 1):
            if peptide not in peptidome:
                peptidome[peptide.replace("I", "L")] = 1

print("known peptides number", len(peptidome))
handle1.close()

filelist = input_file.split(",")
handle_list = []
for f in filelist:
    handle_list.append(SeqIO.parse(f, 'fasta'))  # mutproteins.fasta

output = open(output_file, 'w')

var_peptidome = OrderedDict()

for h in handle_list:
    for record in h:
        proseq = record.seq
        descrip = record.description
        if len(proseq) >= min:
            peptide_list = TRYPSIN(str(proseq), cleavageMiss)
            for peptide in peptide_list:
                if len(peptide) in range(min, max + 1):
                    peptide1 = peptide.replace("I", "L")
                    if peptide1 not in peptidome:  # check if the peptides are in canonical protein sequences
                        des_list = descrip.split(":")
                        des_list[0] = header_prefix
                        mut_type = des_list[-1]
                        snp = des_list[-2]
                        if "Missense" in mut_type:
                            try:
                                mut_pos = int(re.findall(r'\d+', snp)[0])
                                index = str(proseq).index(peptide)
                                pos = mut_pos - index
                                des_list.append(str(pos))
                            except IndexError:
                                continue

                        new_description = ":".join(des_list).replace("*", "-")
                        if peptide not in var_peptidome:
                            var_peptidome[peptide] = [new_description]
                        else:
                            var_peptidome[peptide].append(new_description)
    h.close()
    print("file process done")

print("mut peptide numbers", len(var_peptidome))
for pep in var_peptidome.keys():
    acc = ";".join(set(var_peptidome[pep]))
    output.write(">%s\n%s\n" % (acc, pep))

output.close()



"""
    this script will map proteomics identified peptides to genome and output an peptide gff3 file with coordinates.
    The input peptide table should contain minimum two columns, peptide sequence and Ensembl protein ID.
    Three additional files are required:
    1. Ensembl protein db (same one used in database search)
    2. Ensembl GTF annotation
    3. IDmap file with three columns, first column Ensembl Gene ID, second column Ensembl transcript ID and third Ensembl protein ID.
"""

import sys
import os
import getopt
import numpy as np
import re
from Bio import SeqIO


class EXON(object):
    def __init__(self, number=0, gene=None, variant=None, chr=None, strand=None, start=0, end=0, length=0,
                 trans_start=0, trans_end=0):
        self.gene = gene
        self.variant = variant
        self.number = number
        self.start = start  # chromosome start coordinate
        self.end = end  # chromosome end coordinate
        self.strand = strand
        self.chr = chr
        self.trans_start = trans_start
        self.trans_end = trans_end

    def length(self):
        self.length = self.trans_end - self.trans_start + 1


def cal_trans_pos(exon_list):  # calculate transcript position of exon start & end, exon_list is a list of exon objects
    strand = exon_list[0].strand
    if strand == "+":
        new_exon_list = sorted(exon_list, key=lambda x: x.start)
    else:
        new_exon_list = sorted(exon_list, key=lambda x: x.start, reverse=True)

    sumExonlength = 0
    for exon in new_exon_list:
        exon_length = exon.end - exon.start + 1
        exon.trans_start = 1 + sumExonlength
        exon.trans_end = exon.trans_start + exon_length - 1
        sumExonlength += exon_length

    return new_exon_list


def get_pep_cor(exon_object_list, n1,
                n2):  # return peptide's chromosome start and end cor given peptide's trans_start (n1) and trans_end (n2)
    pep_chr = ""
    pep_strand = ""
    pep_chr_start = 0
    pep_chr_end = 0
    pep_start_exon = 0
    pep_end_exon = 0
    for i in range(len(exon_object_list)):
        exon = exon_object_list[i]
        if exon.trans_end >= n1 >= exon.trans_start:
            pep_chr = exon.chr
            pep_strand = exon.strand
            pep_start_exon = i + 1
            if pep_strand == '+':
                pep_chr_start = exon.start + (n1 - exon.trans_start)
            else:
                pep_chr_end = exon.end - (n1 - exon.trans_start)

        if n2 <= exon.trans_end and n2 >= exon.trans_start:
            pep_chr = exon.chr
            pep_strand = exon.strand
            pep_end_exon = i + 1
            if pep_strand == '+':
                pep_chr_end = exon.start + (n2 - exon.trans_start)
            else:  # chr_cor of n2 is pep_chr_start
                pep_chr_start = exon.end - (n2 - exon.trans_start)

    return pep_chr, pep_strand, pep_chr_start, pep_chr_end, pep_start_exon, pep_end_exon


def parse_gtf(infile):
    dic = {}
    with open(infile, "r") as infile_object:
        for line in infile_object:
            if line[0] != "#":  # skip lines commmented out
                row = line.strip().split("\t")
                if row[2] == "CDS":
                    attri_list = row[8].split(";")
                    transID = ""
                    exon = EXON(start=int(row[3]), end=int(row[4]), chr=row[0], strand=row[6])
                    for attri in attri_list:
                        if "transcript_id" in attri:
                            transID = attri.strip().replace("transcript_id ", "").replace('\"', "")

                    if transID not in dic:
                        dic[transID] = [exon]
                    else:
                        dic[transID].append(exon)
    return dic


################  Comand-line arguments ################
# default peptide sequence in first column, protein ID in second column
pep_col = 1
prot_col = 2

if len(sys.argv[1:]) <= 1:  ### Indicates that there are insufficient number of command-line arguments
    print("Warning! wrong command, please read the mannual")
    print(
        "Example: python map_peptide2genome.py --input peptideTable.txt --gtf Homo_sapiens.GRCh37.90.gtf --fasta Homo_sapiens.GRCh37.90.pep.all.fa  --IDmap Ensembl90_IDmap.txt --output output_filename")
else:
    options, remainder = getopt.getopt(sys.argv[1:], '',
                                       ['input=', 'gtf=', 'fasta=', 'IDmap=', 'output=', 'pep_col=', 'prot_col='])
    for opt, arg in options:
        if opt == '--input':
            input_file = arg
        elif opt == '--gtf':
            gtf_file = arg
        elif opt == '--fasta':
            fasta_file = arg
        elif opt == '--IDmap':
            IDmap_file = arg
        elif opt == '--output':
            output_file = arg
        elif opt == '--pep_col':
            pep_col = int(arg)
        elif opt == '--prot_col':
            prot_col = int(arg)
        else:
            print("Warning! Command-line argument: %s not recognized. Exiting..." % opt)
            sys.exit()

print("reading GTF input file")
feature_dic = parse_gtf(gtf_file)
print("number of unique transcripts in GTF file", len(feature_dic))

IDlist_input = open(IDmap_file, "r")
id_dic = {}
for line in IDlist_input:
    row = line.strip().split("\t")
    if len(row) == 3:
        enst = row[1]
        ensp = row[2]
        if ensp not in id_dic:
            id_dic[ensp] = enst
IDlist_input.close()
print("number of unique ENSP IDs in ID table", len(id_dic))

pep_dic = {}
input = open(input_file,
             'r')  # peptide table with two columns, peptide sequence in first column, protein ID in second column

input.readline()
for line in input:
    row = line.strip().split("\t")
    pep = re.sub("[\W\d]", "", row[pep_col - 1].strip())
    acc = row[prot_col - 1].split(";")[0]  # in case there are multiple IDs
    if pep not in pep_dic:
        pep_dic[pep] = acc

input.close()

output = open(output_file, 'w')

seq_dic = SeqIO.index(fasta_file, 'fasta')
print("number of unique protein sequences in fasta file", len(seq_dic))

non_mapped_pep = 0

for peptide, ensp in pep_dic.items():
    enst = id_dic[ensp]
    try:
        exons = feature_dic[enst]
    except KeyError:
        non_mapped_pep += 1
        continue;

    aa_seq = str(seq_dic[ensp].seq)
    pep_index = aa_seq.index(peptide)

    pep_trans_start = 3 * pep_index + 1
    pep_trans_end = pep_trans_start + 3 * len(peptide) - 1

    exons = cal_trans_pos(exons)

    # print pep_trans_start,pep_trans_end
    pep_chr, pep_strand, pep_chr_start, pep_chr_end, pep_start_exon, pep_end_exon = get_pep_cor(exons, pep_trans_start,
                                                                                                pep_trans_end)

    # handle exceptions
    if pep_chr_start > pep_chr_end:
        non_mapped_pep += 1
        # print peptide,ensp,enst
        continue;
    if pep_chr_start <= 0:
        non_mapped_pep += 1
        # print peptide,ensp,enst,pep_trans_start,pep_trans_end
        continue;

    # print pep_chr_start,pep_chr_end
    # print pep_start_exon,pep_end_exon
    pep_chr = "chr" + pep_chr.replace("MT", "M")
    if pep_start_exon == pep_end_exon:  # if peptide map to one exon
        gff_format_line1 = [pep_chr, "MS", "mRNA", pep_chr_start, pep_chr_end, ".", pep_strand, ".", "ID=" + peptide]
        gff_format_line2 = [pep_chr, "MS", "CDS", pep_chr_start, pep_chr_end, ".", pep_strand, "0", "Parent=" + peptide]
        output.write("\t".join(map(str, gff_format_line1)) + "\n")
        output.write("\t".join(map(str, gff_format_line2)) + "\n")
    elif abs(pep_start_exon - pep_end_exon) == 1:  # if it is a splice junction peptide spanning over two exons
        if pep_strand == "+":
            gff_format_line1 = [pep_chr, "MS", "mRNA", pep_chr_start, pep_chr_end, ".", pep_strand, ".",
                                "ID=" + peptide]
            gff_format_line2 = [pep_chr, "MS", "CDS", pep_chr_start, exons[pep_start_exon - 1].end, ".", pep_strand,
                                "0", "Parent=" + peptide]
            gff_format_line3 = [pep_chr, "MS", "CDS", exons[pep_end_exon - 1].start, pep_chr_end, ".", pep_strand, ".",
                                "Parent=" + peptide]
        else:
            gff_format_line1 = [pep_chr, "MS", "mRNA", pep_chr_start, pep_chr_end, ".", pep_strand, ".",
                                "ID=" + peptide]
            gff_format_line2 = [pep_chr, "MS", "CDS", pep_chr_start, exons[pep_end_exon - 1].end, ".", pep_strand, "0",
                                "Parent=" + peptide]
            gff_format_line3 = [pep_chr, "MS", "CDS", exons[pep_start_exon - 1].start, pep_chr_end, ".", pep_strand,
                                ".", "Parent=" + peptide]

        output.write("\t".join(map(str, gff_format_line1)) + "\n")
        output.write("\t".join(map(str, gff_format_line2)) + "\n")
        output.write("\t".join(map(str, gff_format_line3)) + "\n")
    elif abs(pep_start_exon - pep_end_exon) > 1:  # if peptide span multiple exons,rare case!
        if pep_strand == "+":
            gff_format_line1 = [pep_chr, "MS", "mRNA", pep_chr_start, pep_chr_end, ".", pep_strand, ".",
                                "ID=" + peptide]
            gff_format_line2 = [pep_chr, "MS", "CDS", pep_chr_start, exons[pep_start_exon - 1].end, ".", pep_strand,
                                "0", "Parent=" + peptide]
            gff_format_line3 = [pep_chr, "MS", "CDS", exons[pep_end_exon - 1].start, pep_chr_end, ".", pep_strand, ".",
                                "Parent=" + peptide]
        else:
            gff_format_line1 = [pep_chr, "MS", "mRNA", pep_chr_start, pep_chr_end, ".", pep_strand, ".",
                                "ID=" + peptide]
            gff_format_line2 = [pep_chr, "MS", "CDS", pep_chr_start, exons[pep_end_exon - 1].end, ".", pep_strand, "0",
                                "Parent=" + peptide]
            gff_format_line3 = [pep_chr, "MS", "CDS", exons[pep_start_exon - 1].start, pep_chr_end, ".", pep_strand,
                                ".", "Parent=" + peptide]

        output.write("\t".join(map(str, gff_format_line1)) + "\n")
        output.write("\t".join(map(str, gff_format_line2)) + "\n")
        for k in range(min(pep_start_exon, pep_end_exon) + 1, max(pep_start_exon, pep_end_exon)):
            gff_format_line = [pep_chr, "MS", "CDS", exons[k - 1].start, exons[k - 1].end, ".", pep_strand, ".",
                               "Parent=" + peptide]
            output.write("\t".join(map(str, gff_format_line)) + "\n")

        output.write("\t".join(map(str, gff_format_line3)) + "\n")

output.close()
print("total number of unique peptides", len(pep_dic))
print("total number of unmapped peptides", non_mapped_pep)



#!/usr/bin/env python3

"""
This is the main tool that give access to all commands and options provided by the py-pgatk

@author ypriverol

"""
import click

from pypgatk.commands import ensembl_downloader as ensembl_downloader_cmd
from pypgatk.commands import cosmic_downloader as cosmic_downloader_cmd
from pypgatk.commands import cbioportal_downloader as cbioportal_downloader_cmd
from pypgatk.commands import cosmic_to_proteindb as cosmic_to_proteindb_cmd
from pypgatk.commands import cbioportal_to_proteindb as cbioportal_to_proteindb_cmd
from pypgatk.commands import threeframe_translation as threeframe_translation_cmd
from pypgatk.commands import vcf_to_proteindb as vcf_to_proteindb_cmd
from pypgatk.commands import dnaseq_to_proteindb as dnase_to_proteindb_cmd
from pypgatk.commands import proteindb_decoy as proteindb_decoy_cmd
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


# Cli returns command line requests
@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    """This is the main tool that give access to all commands and options provided by the pypgatk"""


cli.add_command(ensembl_downloader_cmd.ensembl_downloader)
cli.add_command(cbioportal_downloader_cmd.cbioportal_downloader)
cli.add_command(cosmic_downloader_cmd.cosmic_downloader)
cli.add_command(cosmic_to_proteindb_cmd.cosmic_to_proteindb)
cli.add_command(cbioportal_to_proteindb_cmd.cbioportal_to_proteindb)
cli.add_command(threeframe_translation_cmd.threeframe_translation)
cli.add_command(vcf_to_proteindb_cmd.vcf_to_proteindb)
cli.add_command(dnase_to_proteindb_cmd.dnaseq_to_proteindb)
cli.add_command(proteindb_decoy_cmd.generate_database)




if __name__ == "__main__":
    cli()



import base64
import json

import requests

from pypgatk.toolbox.general import ParameterConfiguration, check_create_folders


class CosmicDownloadService(ParameterConfiguration):
    CONFIG_KEY_DATA_DOWNLOADER = 'cosmic_data'
    CONFIG_OUTPUT_DIRECTORY = 'output_directory'
    CONFIG_COSMIC_SERVER = 'cosmic_server'
    CONFIG_COSMIC_FTP_URL = 'cosmic_ftp'
    CONFIG_COSMIC_FTP_USER = "cosmic_user"
    CONFIG_COSMIC_FTP_PASSWORD = "cosmic_password"
    CONFIG_COSMIC_MUTATIONS_URL = "mutations_url"
    CONFIG_COSMIC_MUTATIONS_FILE = "mutations_file"
    CONFIG_COSMIC_CDS_GENES_FILE = "all_cds_genes_file"

    def __init__(self, config_file, pipeline_arguments):
        """
        Init the class with the specific parameters.
        :param config_file configuration file
        :param pipeline_arguments pipelines arguments
        """
        super(CosmicDownloadService, self).__init__(self.CONFIG_KEY_DATA_DOWNLOADER, config_file, pipeline_arguments)

        if self.CONFIG_OUTPUT_DIRECTORY in self.get_pipeline_parameters():
            self._local_path_cosmic = self.get_pipeline_parameters()[self.CONFIG_OUTPUT_DIRECTORY]
        else:
            self._local_path_cosmic = self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][
                self.CONFIG_OUTPUT_DIRECTORY]

        self._cosmic_token = base64.b64encode("{}:{}".format(self.get_pipeline_parameters()[self.CONFIG_COSMIC_FTP_USER], self.get_pipeline_parameters()[self.CONFIG_COSMIC_FTP_PASSWORD])
                                              .encode()).decode('utf-8')

        self.prepare_local_cosmic_repository()

    def prepare_local_cosmic_repository(self):
        self.get_logger().debug("Preparing local cbioportal repository, root folder - '{}'".format(
            self.get_local_path_root_cosmic_repo()))
        check_create_folders([self.get_local_path_root_cosmic_repo()])
        self.get_logger().debug(
            "Local path for cbioportal Release - '{}'".format(self.get_local_path_root_cosmic_repo()))

    def get_local_path_root_cosmic_repo(self):
        return self._local_path_cosmic

    def download_mutation_file(self):
        """
        This function will download the mutations file from Cosmic Database.
        :return: None
        """

        mutation_output_file = "{}/{}".format(self.get_local_path_root_cosmic_repo(), self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_COSMIC_SERVER][self.CONFIG_COSMIC_MUTATIONS_FILE])
        cds_genes_output_file = "{}/{}".format(self.get_local_path_root_cosmic_repo(), self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_COSMIC_SERVER][self.CONFIG_COSMIC_CDS_GENES_FILE])

        server = self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_COSMIC_SERVER][
            self.CONFIG_COSMIC_FTP_URL]
        cosmic_version = self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_COSMIC_SERVER][
            self.CONFIG_COSMIC_MUTATIONS_URL]
        mutation_file = self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_COSMIC_SERVER][
            self.CONFIG_COSMIC_MUTATIONS_FILE]

        all_cds_gene_file = self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_COSMIC_SERVER][
            self.CONFIG_COSMIC_CDS_GENES_FILE]

        mutation_url = "{}/{}/{}".format(server, cosmic_version, mutation_file)
        cds_gene_url = "{}/{}/{}".format(server, cosmic_version, all_cds_gene_file)

        token = "Basic {}".format(self._cosmic_token)
        self.download_file_cosmic(mutation_url, mutation_output_file, token)
        self.download_file_cosmic(cds_gene_url, cds_genes_output_file, token)

    def download_file_cosmic(self, url, local_file, token):
        """
        Download file from cosmic repository using requests
        :param url: url of the file to be download
        :param local_file: local file
        :param token: token to be used
        :return:
        """

        response = requests.get(url, stream=True, headers={'Authorization': token})
        if response.status_code == 200:
            url = json.loads(response.text)['url']
            msg = "Downloading file from url '{}'".format(url)
            self.get_logger().debug(msg)

            response = requests.get(url, stream=True)
            if response.status_code == 200:
                with open(local_file, 'wb') as f:
                    f.write(response.content)
                    msg = "Download Finish for file '{}'".format(local_file)
                    self.get_logger().debug(msg)



import click

from pypgatk.commands.utils import print_help
from pypgatk.ensembl.ensembl import EnsemblDataService


@click.command('vcf-to-proteindb', short_help="Generate peptides based on DNA variants from ENSEMBL VEP VCF files")
@click.option('--config_file', '-c', help='Configuration to perform conversion between ENSEMBL Files',
              default='config/ensembl_config.yaml')
@click.option('-f', '--input_fasta', help='Path to the transcript sequence')
@click.option('-v', '--vep_annotated_vcf', help='Path to the vep annotated VCF file')
@click.option('-g', '--gene_annotations_gtf', help='Path to the gene annotations file')
@click.option('-t', '--translation_table', default=1, type=int, help="Translation table (Default 1) ")
@click.option('-m', '--mito_translation_table', default=2, type=int, help='Mito_trans_table (default 2)')
@click.option('-p', '--var_prefix', default="var", help="String to add before the variant peptides")
@click.option('--report_ref_seq', help='In addition to var peps, also report all ref peps', is_flag=True)
@click.option('-o', '--output_proteindb', default="peptide-database.fa", help="Output file name, exits if already exists")
@click.option('--annotation_field_name', default="CSQ",
              help="Annotation field name found in the INFO column, e.g CSQ or vep")
@click.option('--af_field', default="", help="field name in the VCF INFO column to use for filtering on AF, (Default None)")
@click.option('--af_threshold', default=0.01, help='Minium AF threshold for considering common variants')
@click.option('--transcript_index', default=3, type=int,
              help='Index of transcript ID in the annotated columns (separated by |)')
@click.option('--consequence_index', default=1, type=int,
              help='Index of consequence in the annotated columns (separated by |)')
@click.option('--exclude_biotypes', default='', help="Excluded Biotypes")
@click.option('--exclude_consequences',
              default='downstream_gene_variant, upstream_gene_variant, intergenic_variant, intron_variant, synonymous_variant',
              help="Excluded Consequences", show_default=True)
@click.option('-s', '--skip_including_all_cds',
              help="by default any transcript that has a defined CDS will be used, this option disables this features instead it only depends on the biotypes",
              is_flag=True)
@click.option('--include_biotypes', default='protein_coding,polymorphic_pseudogene,non_stop_decay,nonsense_mediated_decay,IG_C_gene,IG_D_gene,IG_J_gene,IG_V_gene,TR_C_gene,TR_D_gene,TR_J_gene,TR_V_gene,TEC', help="Only genes/transcripts (depending on --biotype_str) from these biotypes are considered for trasnaltion, in addition CDS transcripts unless --skip_including_all_cds is given")
@click.option('--include_consequences', default='all', help="included_consequences, default all")
@click.option('--biotype_str', default='transcript_biotype',
              help='string used to identify gene/transcript biotype in the gtf file.')
@click.option('--ignore_filters',
              help="enabling this option causes or variants to be parsed. By default only variants that have not failed any filters will be processed (FILTER column is PASS, None, .) or if the filters are subset of the accepted filters. (default is False)",
              is_flag=True)
@click.option('--accepted_filters', default='', help="Accepted filters for variant parsing")
@click.pass_context
def vcf_to_proteindb(ctx, config_file, input_fasta, vep_annotated_vcf, gene_annotations_gtf, translation_table,
                     mito_translation_table,
                     var_prefix, report_ref_seq, output_proteindb, annotation_field_name,
                     af_field, af_threshold, transcript_index, consequence_index, exclude_biotypes,
                     exclude_consequences, skip_including_all_cds, include_biotypes, include_consequences, biotype_str,
                     ignore_filters, accepted_filters):
    if input_fasta is None or vep_annotated_vcf is None or gene_annotations_gtf is None:
        print_help()

    pipeline_arguments = {}
    pipeline_arguments[EnsemblDataService.MITO_TRANSLATION_TABLE] = mito_translation_table
    pipeline_arguments[EnsemblDataService.TRANSLATION_TABLE] = translation_table
    pipeline_arguments[EnsemblDataService.HEADER_VAR_PREFIX] = var_prefix
    pipeline_arguments[EnsemblDataService.REPORT_REFERENCE_SEQ] = report_ref_seq
    pipeline_arguments[EnsemblDataService.PROTEIN_DB_OUTPUT] = output_proteindb
    pipeline_arguments[EnsemblDataService.ANNOTATION_FIELD_NAME] = annotation_field_name
    pipeline_arguments[EnsemblDataService.AF_FIELD] = af_field
    pipeline_arguments[EnsemblDataService.AF_THRESHOLD] = af_threshold
    pipeline_arguments[EnsemblDataService.TRANSCRIPT_INDEX] = transcript_index
    pipeline_arguments[EnsemblDataService.CONSEQUENCE_INDEX] = consequence_index
    pipeline_arguments[EnsemblDataService.EXCLUDE_BIOTYPES] = exclude_biotypes
    pipeline_arguments[EnsemblDataService.EXCLUDE_CONSEQUENCES] = exclude_consequences
    pipeline_arguments[EnsemblDataService.SKIP_INCLUDING_ALL_CDS] = skip_including_all_cds
    pipeline_arguments[EnsemblDataService.INCLUDE_BIOTYPES] = include_biotypes
    pipeline_arguments[EnsemblDataService.INCLUDE_CONSEQUENCES] = include_consequences
    pipeline_arguments[EnsemblDataService.BIOTYPE_STR] = biotype_str
    pipeline_arguments[EnsemblDataService.IGNORE_FILTERS] = ignore_filters
    pipeline_arguments[EnsemblDataService.ACCEPTED_FILTERS] = accepted_filters

    ensembl_data_service = EnsemblDataService(config_file, pipeline_arguments)
    ensembl_data_service.vcf_to_proteindb(vep_annotated_vcf, input_fasta, gene_annotations_gtf)



import click

from pypgatk.cgenomes.cgenomes_proteindb import CancerGenomesService
from pypgatk.commands.utils import print_help


@click.command('cbioportal-to-proteindb', short_help='Command to translate cbioportal mutation data into proteindb')
@click.option('--config_file',
              '-c',
              help='Configuration for cbioportal to proteindb tool',
              default='config/cbioportal_config.yaml')
@click.option('-in', '--input_mutation', help='Cbioportal mutation file')
@click.option('-fa', '--input_cds', help='CDS genes from ENSEMBL database')
@click.option('-out', '--output_db', help='Protein database including all the mutations')
@click.option('-t', '--tissue_type', default = 'all', help='Limit mutations to tissue types considered for generating proteinDBs, by default mutations from all tissue types are considered')
@click.option('-s', '--split_by_tissue_type', help='Use this flag to generate a proteinDB per tissue type as specified in the Primary site column, default is False', is_flag=True)
@click.option('-cl', '--clinical_sample_file', help='File to get tissue type from for the samples in input_mutation file')

@click.pass_context
def cbioportal_to_proteindb(ctx, config_file, input_mutation, input_cds, output_db, clinical_sample_file, tissue_type, split_by_tissue_type):
    if input_mutation is None or input_cds is None or output_db is None:
        print_help()

    pipeline_arguments = {CancerGenomesService.CONFIG_CANCER_GENOMES_MUTATION_FILE: input_mutation,
                          CancerGenomesService.CONFIG_COMPLETE_GENES_FILE: input_cds,
                          CancerGenomesService.CONFIG_OUTPUT_FILE: output_db,
                          CancerGenomesService.CLINICAL_SAMPLE_FILE: clinical_sample_file,
                          CancerGenomesService.TISSUE_TYPE: tissue_type,
                          CancerGenomesService.SPLIT_BY_TISSUE_TYPE: split_by_tissue_type}

    cosmic_to_proteindb_service = CancerGenomesService(config_file, pipeline_arguments)
    cosmic_to_proteindb_service.cbioportal_to_proteindb()



import click

CONTEXT_SETTINGS = {
    'help_option_names': ['-h', '--help'],
}


def echo_success(text, nl=True):
    click.secho(text, fg='cyan', bold=True, nl=nl)


def echo_failure(text, nl=True):
    click.secho(text, fg='red', bold=True, nl=nl)


def echo_warning(text, nl=True):
    click.secho(text, fg='yellow', bold=True, nl=nl)


def echo_waiting(text, nl=True):
    click.secho(text, fg='magenta', bold=True, nl=nl)


def echo_info(text, nl=True):
    click.secho(text, bold=True, nl=nl)


def print_help():
    """
    Print the help of the tool
    :return:
    """
    ctx = click.get_current_context()
    click.echo(ctx.get_help())
    ctx.exit()




"""
This module contains Ensembl models used as Entities and DAO/Services
"""

import json


class Species:
    def __init__(self, ensembl_species_entry):
        self.__ensembl_species_entry = ensembl_species_entry

    def get_logger(self):
        return self.__logger

    def get_value_for_key_or_default(self, key, default_value='-not_available-'):
        if key in self.get_ensembl_species_entry():
            return self.get_ensembl_species_entry()[key]
        return default_value

    def get_ensembl_species_entry(self):
        return self.__ensembl_species_entry

    def get_division(self):
        return self._get_value_for_key_or_default('division')

    def get_ncbi_taxonomy_id(self):
        return self._get_value_for_key_or_default('taxon_id')

    def get_name(self):
        return self._get_value_for_key_or_default('name')

    def get_ensembl_release(self):
        return self._get_value_for_key_or_default('release')

    def get_display_name(self):
        return self._get_value_for_key_or_default('display_name')

    def get_assembly_accession(self):
        return self._get_value_for_key_or_default('accession')

    def get_strain_collection(self):
        if self._get_value_for_key_or_default('strain_collection', 'null') == 'null':
            return None
        return self._get_value_for_key_or_default('strain_collection')

    def get_common_name(self):
        return self._get_value_for_key_or_default('common_name')

    def get_strain(self):
        return self._get_value_for_key_or_default('strain')

    def get_aliases(self):
        return self._get_value_for_key_or_default('aliases', [])

    def get_groups(self):
        return self._get_value_for_key_or_default('groups', [])

    def get_assembly(self):
        return self._get_value_for_key_or_default('assembly')

    def __str__(self):
        return json.dumps(self.get_ensembl_species_entry())


class SpeciesService:
    def __init__(self, species_data):
        # I've changed this, we store the original species data, and then we offer two different views
        self.__ensembl_species_data_raw = species_data
        self.__ensembl_species_data_dao = None
        self.__index_by_taxonomy_id = None
        self.__index_by_assembly = None

    def __index_data_for_property(self, data, property_getter):
        """
        Given an iterable data container, and a property getter to run on every object of that container, it returns a
        dictionary where the key is the property value for a particular data object part of the data collection
        :param data: iterable of data objects to index
        :param property_getter: property on which the index should be created, this property is a member function to a
        getter for that property
        :return: a dictionary of the given data objects where the key is the indexed property
        """
        self._get_logger().debug("Creating index on getter '{}' for #{} entries".format(property_getter, len(data)))
        return {property_getter(data_item): data_item for data_item in data}

    def __index_data_by_taxonomy_ensembl_special_case(self, data):
        indexed_data = {}
        for data_item in data:
            if data_item.get_ncbi_taxonomy_id() not in indexed_data:
                indexed_data[data_item.get_ncbi_taxonomy_id()] = data_item
            else:
                # Keep the one with aliases, that seems to be the main one for Ensembl
                if data_item.get_aliases():
                    if indexed_data[data_item.get_ncbi_taxonomy_id()].get_aliases():
                        self._get_logger().error("ENSEMBL SPECIES INDEXING ERROR, already existing species entry '{}' "
                                                 "will not be replaced by non-empty aliases entry '{}'"
                                                 .format(str(indexed_data[data_item.get_ncbi_taxonomy_id()]),
                                                         str(data_item)))
                    else:
                        self._get_logger().warning("ENSEMBL SPECIES INDEX REPLACEMENT, of existing entry '{}' "
                                                   "by new entry '{}'"
                                                   .format(str(indexed_data[data_item.get_ncbi_taxonomy_id()]),
                                                           str(data_item)))
                        indexed_data[data_item.get_ncbi_taxonomy_id()] = data_item
                else:
                    self._get_logger().warning("ALREADY INDEXED ENSEMBL SPECIES ENTRY '{}' "
                                               "WILL NOT BE REPLACED by '{}'"
                                               .format(str(indexed_data[data_item.get_ncbi_taxonomy_id()]),
                                                       str(data_item)))
        return indexed_data

    def _get_logger(self):
        return self.__logger

    def _get_species_data_dao(self):
        """
        This method creates the DAO version of the ensembl species data, it could have been implemented as a generator
        but I want those DAO objects to be reused by the Python reference system, so having them in multiple indexes
        does not require multiple times the same amount of memory
        :return: a list of DAO accessors (Species) to the given RAW ensembl species data
        """
        if self.__ensembl_species_data_dao is None:
            self.__ensembl_species_data_dao = \
                [Species(species_entry) for species_entry in self.get_species_data()['species']]
        return self.__ensembl_species_data_dao

    def _get_index_taxonomy_id(self):
        """
        Build the index for species data by taxonomy ID
        :return: ensembl species data indexed by taxonomy ID
        """
        if self.__index_by_taxonomy_id is None:
            self.__index_by_taxonomy_id = \
                self.__index_data_by_taxonomy_ensembl_special_case(self._get_species_data_dao())
            # Generic and beautiful old way of building the index
            # self.__index_data_for_property(self._get_species_data_dao(), Species.get_ncbi_taxonomy_id)
        return self.__index_by_taxonomy_id

    def _get_index_assembly(self):
        """
        Build the index for species data by Assembly
        :return: ensembl species data indexd by assembly
        """
        if not self.__index_by_assembly:
            self.__index_by_assembly = self.__index_data_for_property(self._get_species_data_dao(),
                                                                      Species.get_assembly)
        return self.__index_by_assembly

    def get_species_data(self):
        """
        Get the Ensembl species data unprocessed, as it came from the Ensembl REST API
        :return: raw ensembl species data
        """
        return self.__ensembl_species_data_raw

    def get_species_entry_for_taxonomy_id(self, taxonomy_id):
        """
        Given a taxonomy ID, get its Ensembl species entry
        :param taxonomy_id: taxonomy ID
        :return: the species entry or None if not found
        """
        self._get_logger().debug("get_species_entry_for_taxonomy_id ---> taxonomy id '{}'".format(taxonomy_id))
        if taxonomy_id in self._get_index_taxonomy_id():
            return self._get_index_taxonomy_id()[taxonomy_id]
        return None

    def get_species_entry_for_assembly(self, assembly):
        """
        Given an Assembly, get its Ensembl species entry
        :param assembly: assembly
        :return: the species entry or None if not found
        """
        self._get_logger().debug("get_species_entry_for_assembly '{}'".format(assembly))
        if assembly in self._get_index_assembly():
            return self._get_index_assembly()[assembly]
        return None

    def count_ensembl_species(self):
        return len(self._get_species_data_dao())


if __name__ == '__main__':
    print("ERROR: This script is part of a pipeline collection and it is not meant to be run in stand alone mode")



"""
This module implements some useful functions for the pipeline runner.

@author ypriverol
"""
import logging
import os
import json
import shutil
import subprocess
from urllib import error, request
from urllib.error import URLError, ContentTooShortError
import yaml

# Logging defaults
from requests import HTTPError

from pypgatk.toolbox.exceptions import ToolBoxException


REMAINING_DOWNLOAD_TRIES = 4

class ParameterConfiguration:
    """
    This class is a helper class for those submodules having to manage configuration files themselves, that are specific
    to them
    """

    _logger_formatters = {
    "DEBUG": "%(asctime)s [%(levelname)7s][%(name)28s][%(module)18s, %(lineno)4s] %(message)s",
    "INFO": "%(asctime)s [%(levelname)7s][%(name)28s] %(message)s"
    }
    _log_level = 'DEBUG'
    
    _CONFIG_LOGGER = 'logger'
    _CONFIG_LOGGER_FORMATTER = 'formatters'
    _CONFIG_LOGGER_LEVEL = 'loglevel'

    def __init__(self, root_config_name, yaml_configuration_file, pipeline_parameters):
        """
        This function creates a parameter structure from a yaml config file and the pipeline paramters provdided
        in the commandline
        :param yaml_configuration_file: yaml config file
        :param pipeline_parameters: commandline parameters.
        """

        self._ROOT_CONFIG_NAME = root_config_name

        self._configuration_file = yaml_configuration_file
        with open(yaml_configuration_file, 'r') as f:
            self._default_params = yaml.load(f.read(), Loader=yaml.FullLoader)
        self._pipeline_parameters = pipeline_parameters

        # Prepare Logging subsystem
        if self._ROOT_CONFIG_NAME in self._default_params:
            if self._CONFIG_LOGGER in self._default_params[self._ROOT_CONFIG_NAME]:
                if self._CONFIG_LOGGER_LEVEL in self._default_params[self._ROOT_CONFIG_NAME][self._CONFIG_LOGGER]:
                    self._log_level = self._default_params[self._ROOT_CONFIG_NAME][self._CONFIG_LOGGER][
                        self._CONFIG_LOGGER_LEVEL]
                if self._CONFIG_LOGGER_FORMATTER in self._default_params[self._ROOT_CONFIG_NAME][self._CONFIG_LOGGER]:
                    self._logger_formatters = self._default_params[self._ROOT_CONFIG_NAME][self._CONFIG_LOGGER][
                        self._CONFIG_LOGGER_FORMATTER]

        self._log_handlers = []
        log_handlers_prefix = self._ROOT_CONFIG_NAME + '-'
        log_handlers_extension = '.log'

        self._logger = logging.getLogger(__name__)
        self._logger.setLevel(getattr(logging, self._log_level))
        self._log_files = []
        for llevel, lformat in self._logger_formatters.items():
            logfile = os.path.join(log_handlers_prefix + llevel.lower() + log_handlers_extension)
            lformatter = logging.Formatter(lformat)
            lhandler = logging.FileHandler(logfile, mode='w')
            lhandler.setLevel(getattr(logging, llevel))
            lhandler.setFormatter(lformatter)
            self._log_handlers.append(lhandler)
            # Add the handlers to my own logger
            self._logger.addHandler(lhandler)
            # Keep the path to the log file
            self._log_files.append(logfile)
        self.get_logger().debug("Logging system initialized")

    def get_pipeline_parameters(self):
        return self._pipeline_parameters

    def get_default_parameters(self):
        return self._default_params

    def get_log_handlers(self):
        return self._log_handlers

    def get_logger(self):
        # Get own logger
        return self._logger

    def get_session_log_files(self):
        log_files = []
        # Add the application logs
        log_files.extend(self._log_files)
        return log_files

    def get_logger_for(self, name):
        """
        Create a logger on demand
        :param name: name to be used in the logger
        :return: a new logger on that name
        """
        self.get_logger().debug("Creating logger with name {}".format(name))
        lg = logging.getLogger(name)
        for handler in self.get_log_handlers():
            lg.addHandler(handler)
        lg.setLevel(self._log_level)
        return lg


def read_json(json_file="json_file_not_specified.json"):
    """
    Reads a json file and it returns its object representation, no extra checks
    are performed on the file so, in case anything happens, the exception will
    reach the caller
    :param json_file: path to the file in json format to read
    :return: an object representation of the data in the json file
    """
    with open(json_file) as jf:
        return json.load(jf)


def check_create_folders(folders: str):
    """
    Check if folders exist, create them otherwise
    :param folders: list of folder paths to check
    :return: no return value
    """
    for folder in folders:
        if not os.path.exists(folder):
            try:
                os.makedirs(folder)
            except Exception as e:
                raise ToolBoxException(str(e))
        else:
            if not os.path.isdir(folder):
                raise ToolBoxException("'{}' is not a folder".format(folder))


def clear_cache():
    request.urlcleanup()


def download_file(file_url: str, file_name: str, log: logging) -> str:
    """
     Download file_url and move it to file_name, do nothing if file_name already exists.

    :param log: log to be use.
    :param file_url: file url to be download
    :param file_name: file name where the data will be downloaded
    :return: name of the file if the file can be download.
    """
    if os.path.isfile(file_name):
        return file_name

    if log is not None:
        log = logging

    remaining_download_tries = REMAINING_DOWNLOAD_TRIES
    downloaded_file = None
    while remaining_download_tries > 0:
        try:
            downloaded_file, error_code = request.urlretrieve(file_url, file_name)
            log.debug("File downloaded -- " + downloaded_file)
            break
        except (HTTPError, URLError, ContentTooShortError) as error:
            logging.error("Error downloading -- Incorrect URL or file not found: " + file_url + " on trial no: " + str(
                REMAINING_DOWNLOAD_TRIES - remaining_download_tries))
            log.error("Error code: " + str(error))
            remaining_download_tries = remaining_download_tries - 1
            downloaded_file = None
            continue
        except Exception as error:
            remaining_download_tries = remaining_download_tries - 1
            log.error("Error code: " + str(error))
            downloaded_file = None

    return downloaded_file

    # move the pep file to the desired name
    # if os.path.isfile(downloaded_file):
    #     if os.stat(downloaded_file).st_size > 1000:
    #         shutil.move(downloaded_file, file_name)
    #         logging.debug("File copy to filesystem -- " + downloaded_file)
    #         return file_name
    #     else:
    #         print("Corrupt File (size<1kb): ", file_url)
    #         os.remove(downloaded_file)
    #         return None
    # else:
    #     print("Failed to download the file: ", file_url)
    #     return None


def check_create_folders_overwrite(folders):
    """
    Given a list of folders, this method will create them, overwriting them in case they exist
    :param folders: list of folders to create
    :return: no return value
    :except: if any element in the list of folders is not a folder, an exception will be raised
    """
    invalid_folders = []
    for folder in folders:
        if os.path.exists(folder):
            if not os.path.isdir(folder):
                invalid_folders.append(folder)
    if invalid_folders:
        # If there's any invalid folder, we don't make any change, and we report the situation by raising an exception
        raise ToolBoxException("The following folders ARE NOT FOLDERS - '{}'"
                               .format(invalid_folders))
    for folder in folders:
        try:
            shutil.rmtree(folder)
        except FileNotFoundError as e:
            # It is find if the folder is not there
            pass
    check_create_folders(folders)


def create_latest_symlink(destination_path):
    """
    Create a symlink 'latest' to the given destination_path in its parent folder, i.e. if the given path is
    '/nfs/production/folder', the symlink will be
            /nfs/production/latest -> /nfs/production/folder
    :param destination_path: destination path where the symlink will point to
    :return: no return value
    """
    symlink_path = os.path.join(os.path.dirname(destination_path), 'latest')
    os.symlink(destination_path, symlink_path)


def create_latest_symlink_overwrite(destination_path):
    """
    Create a symlink 'latest' to the given destination_path in its parent folder, i.e. if the given path is
    '/nfs/production/folder', the symlink will be
            /nfs/production/latest -> /nfs/production/folder
    If there already is a 'latest' symlink, it will be overwritten
    :param destination_path: destination path where the symlink will point to
    :return: no return value
    """
    symlink_path = os.path.join(os.path.dirname(destination_path), 'latest')
    if os.path.islink(symlink_path):
        os.unlink(symlink_path)
    os.symlink(destination_path, symlink_path)


def gunzip_files(files):
    """
    Given a list of paths for Gzip compressed files, this method will uncompress them, returning a list with the files
    that could not be gunzipped and the reason why that happened
    :param files: list of paths to files that will be un-compressed
    :return: a list of possible failing to uncompress files
    """
    gunzip_command_template = "gunzip {}"
    files_with_error = []
    for file in files:
        if os.path.isfile(file):
            try:
                gunzip_subprocess = subprocess.Popen(gunzip_command_template.format(file),
                                                     stdout=subprocess.PIPE,
                                                     stderr=subprocess.PIPE,
                                                     shell=True)
                # Timeout, in seconds, is either 10 seconds or the size of the file in MB * 10, e.g. 1MB -> 10 seconds
                file_size_mb = os.path.getsize(file) / (1024 * 1024)
                timeout = max(10, int(file_size_mb))
                (stdout, stderr) = gunzip_subprocess.communicate(timeout=timeout)
                if gunzip_subprocess.poll() is not None:
                    if gunzip_subprocess.returncode != 0:
                        # ERROR - Report this
                        err_msg = "ERROR uncompressing file '{}' output from subprocess STDOUT: {}\nSTDERR: {}" \
                            .format(file, stdout.decode('utf8'), stderr.decode('utf8'))
                        files_with_error.append((file, err_msg))
            except subprocess.TimeoutExpired as e:
                err_msg = "TIMEOUT ERROR uncompressing file '{}', size {}MB, given timeframe of '{}seconds', output from subprocess STDOUT: {}\nSTDERR: {}" \
                    .format(file_size_mb,
                            timeout,
                            file,
                            stdout.decode('utf8'),
                            stderr.decode('utf8'))
                files_with_error.append((file, err_msg))
            except Exception as e:
                err_msg = "UNKNOWN ERROR uncompressing file '{}' ---> {}\nOutput from subprocess STDOUT: {}\nSTDERR: {}" \
                    .format(file,
                            e,
                            stdout.decode('utf8'),
                            stderr.decode('utf8'))
                files_with_error.append((file, err_msg))
        else:
            files_with_error.append((file, "it IS NOT A FILE"))
    return files_with_error


if __name__ == '__main__':
    print("ERROR: This script is part of a pipeline collection and it is not meant to be run in stand alone mode")



import gffutils
import vcf
from Bio import SeqIO
from Bio.Seq import Seq

from pypgatk.toolbox.general import ParameterConfiguration


class EnsemblDataService(ParameterConfiguration):
    CONFIG_KEY_VCF = "ensembl_translation"
    INPUT_FASTA = "input_fasta"
    TRANSLATION_TABLE = "translation_table"
    MITO_TRANSLATION_TABLE = "mito_translation_table"
    HEADER_VAR_PREFIX = "var_prefix"
    REPORT_REFERENCE_SEQ = "report_ref_seq"
    PROTEIN_DB_OUTPUT = "proteindb_output_file"
    ANNOTATION_FIELD_NAME = "annotation_field_name"
    AF_FIELD = "af_field"
    AF_THRESHOLD = "af_threshold"
    TRANSCRIPT_INDEX = "transcript_index"
    CONSEQUENCE_INDEX = "consequence_index"
    EXCLUDE_BIOTYPES = "exclude_biotypes"
    EXCLUDE_CONSEQUENCES = "exclude_consequences"
    SKIP_INCLUDING_ALL_CDS = "skip_including_all_cds"
    INCLUDE_BIOTYPES = "include_biotypes"
    INCLUDE_CONSEQUENCES = "include_consequences"
    BIOTYPE_STR = "biotype_str"
    SKIP_INCLUDING_ALL_CDSS = "skip_including_all_CDSs"
    CONFIG_KEY_DATA = "ensembl_translation"
    NUM_ORFS = "num_orfs"
    NUM_ORFS_COMPLEMENT = "num_orfs_complement"
    EXPRESSION_STR = "expression_str"
    EXPRESSION_THRESH = "expression_thresh"
    IGNORE_FILTERS = "ignore_filters"
    ACCEPTED_FILTERS = "accepted_filters"

    def __init__(self, config_file, pipeline_arguments):
        """
        Init the class with the specific parameters.
        :param config_file configuration file
        :param pipeline_arguments pipelines arguments
        """
        super(EnsemblDataService, self).__init__(self.CONFIG_KEY_DATA, config_file,
                                                 pipeline_arguments)

        self._proteindb_output = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.PROTEIN_DB_OUTPUT]
        if self.PROTEIN_DB_OUTPUT in self.get_pipeline_parameters():
            self._proteindb_output = self.get_pipeline_parameters()[self.PROTEIN_DB_OUTPUT]

        self._translation_table = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.TRANSLATION_TABLE]
        if self.TRANSLATION_TABLE in self.get_pipeline_parameters():
            self._translation_table = self.get_pipeline_parameters()[self.TRANSLATION_TABLE]

        self._mito_translation_table = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.MITO_TRANSLATION_TABLE]
        if self.MITO_TRANSLATION_TABLE in self.get_pipeline_parameters():
            self._mito_translation_table = self.get_pipeline_parameters()[self.MITO_TRANSLATION_TABLE]

        self._header_var_prefix = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.HEADER_VAR_PREFIX]
        if self.HEADER_VAR_PREFIX in self.get_pipeline_parameters():
            self._header_var_prefix = self.get_pipeline_parameters()[self.HEADER_VAR_PREFIX]

        self._report_reference_seq = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.REPORT_REFERENCE_SEQ]
        if self.REPORT_REFERENCE_SEQ in self.get_pipeline_parameters():
            self._report_reference_seq = self.get_pipeline_parameters()[self.REPORT_REFERENCE_SEQ]

        self._annotation_field_name = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.ANNOTATION_FIELD_NAME]
        if self.ANNOTATION_FIELD_NAME in self.get_pipeline_parameters():
            self._annotation_field_name = self.get_pipeline_parameters()[self.ANNOTATION_FIELD_NAME]

        self._af_field = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.AF_FIELD]
        if self.AF_FIELD in self.get_pipeline_parameters():
            self._af_field = self.get_pipeline_parameters()[self.AF_FIELD]

        self._af_threshold = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.AF_THRESHOLD]
        if self.AF_THRESHOLD in self.get_pipeline_parameters():
            self._af_threshold = self.get_pipeline_parameters()[self.AF_THRESHOLD]

        self._transcript_index = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.TRANSCRIPT_INDEX]
        if self.TRANSCRIPT_INDEX in self.get_pipeline_parameters():
            self._transcript_index = self.get_pipeline_parameters()[self.TRANSCRIPT_INDEX]

        self._consequence_index = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.CONSEQUENCE_INDEX]
        if self.CONSEQUENCE_INDEX in self.get_pipeline_parameters():
            self._consequence_index = self.get_pipeline_parameters()[self.CONSEQUENCE_INDEX]

        self._exclude_biotypes = self.get_multiple_options(
            self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][self.EXCLUDE_BIOTYPES])
        if self.EXCLUDE_BIOTYPES in self.get_pipeline_parameters():
            self._exclude_biotypes = self.get_multiple_options(self.get_pipeline_parameters()[self.EXCLUDE_BIOTYPES])

        self._exclude_consequences = self.get_multiple_options(
            self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][self.EXCLUDE_CONSEQUENCES])
        if self.EXCLUDE_CONSEQUENCES in self.get_pipeline_parameters():
            self._exclude_consequences = self.get_multiple_options(
                self.get_pipeline_parameters()[self.EXCLUDE_CONSEQUENCES])

        self._skip_including_all_cds = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.SKIP_INCLUDING_ALL_CDS]
        if self.SKIP_INCLUDING_ALL_CDS in self.get_pipeline_parameters():
            self._skip_including_all_cds = self.get_pipeline_parameters()[self.SKIP_INCLUDING_ALL_CDS]

        self._include_biotypes = self.get_multiple_options(
            self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][self.INCLUDE_BIOTYPES])
        if self.INCLUDE_BIOTYPES in self.get_pipeline_parameters():
            self._include_biotypes = self.get_multiple_options(self.get_pipeline_parameters()[self.INCLUDE_BIOTYPES])

        self._include_consequences = self.get_multiple_options(
            self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][self.INCLUDE_CONSEQUENCES])
        if self.INCLUDE_CONSEQUENCES in self.get_pipeline_parameters():
            self._include_consequences = self.get_multiple_options(
                self.get_pipeline_parameters()[self.INCLUDE_CONSEQUENCES])

        self._biotype_str = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.BIOTYPE_STR]
        if self.BIOTYPE_STR in self.get_pipeline_parameters():
            self._biotype_str = self.get_pipeline_parameters()[self.BIOTYPE_STR]

        self._num_orfs = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][self.NUM_ORFS]
        if self.NUM_ORFS is self.get_pipeline_parameters():
            self._num_orfs = self.get_pipeline_parameters()[self.NUM_ORFS]

        self._num_orfs_complement = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.NUM_ORFS_COMPLEMENT]
        if self.NUM_ORFS_COMPLEMENT is self.get_pipeline_parameters():
            self._num_orfs_complement = self.get_pipeline_parameters()[self.NUM_ORFS_COMPLEMENT]

        self._expression_str = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.EXPRESSION_STR]
        if self.EXPRESSION_STR is self.get_pipeline_parameters():
            self._expression_str = self.get_pipeline_parameters()[self.EXPRESSION_STR]

        self._expression_thresh = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.EXPRESSION_THRESH]
        if self.EXPRESSION_THRESH is self.get_pipeline_parameters():
            self._expression_thresh = self.get_pipeline_parameters()[self.EXPRESSION_THRESH]

        self._ignore_filters = self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][
            self.IGNORE_FILTERS]
        if self.IGNORE_FILTERS in self.get_pipeline_parameters():
            self._ignore_filters = self.get_pipeline_parameters()[self.IGNORE_FILTERS]

        self._accepted_filters = self.get_multiple_options(
            self.get_default_parameters()[self.CONFIG_KEY_DATA][self.CONFIG_KEY_VCF][self.ACCEPTED_FILTERS])
        if self.ACCEPTED_FILTERS in self.get_pipeline_parameters():
            self._accepted_filters = self.get_multiple_options(
                self.get_pipeline_parameters()[self.ACCEPTED_FILTERS])

    def three_frame_translation(self, input_fasta):
        """
        This function translate a transcriptome into a 3'frame translation protein sequence database
        :param input_fasta: fasta input file
        :return:
        """

        input_handle = open(input_fasta, 'r')
        output_handle = open(self._proteindb_output, 'rw')

        for record in SeqIO.parse(input_handle, 'fasta'):
            seq = record.seq
            RF1 = seq.translate(table=self._translation_table)
            RF2 = seq[1::].translate(table=self._translation_table)
            RF3 = seq[2::].translate(table=self._translation_table)

            if record.id == "":
                print("skip entries without id", record.description)
                continue
            output_handle.write("%s\n%s\n" % ('>' + record.id + '_RF1', RF1))
            output_handle.write("%s\n%s\n" % ('>' + record.id + '_RF2', RF2))
            output_handle.write("%s\n%s\n" % ('>' + record.id + '_RF3', RF3))

    @staticmethod
    def get_multiple_options(options_str: str):
        """
        This method takes an String like option1, option2, ... and produce and array [option1, option2,... ]
        :param options_str:
        :return: Array
        """
        return list(map(lambda x: x.strip(), options_str.split(",")))

    @staticmethod
    def check_overlap(var_start, var_end, features_info=[[0, 1, 'type']]):
        """
        This function returns true when the variant overlaps any of the features
        :param var_start: Start location
        :param var_end: End location
        :param features_info: Feature information
        :return:
        """
        if var_start == -1:
            return True
        # check if the var overlaps any of the features
        for feature_pos in features_info:
            pep_start = feature_pos[0]
            pep_end = feature_pos[1]
            if var_start <= pep_start <= var_end:  # fully contained or partial overlap from the end
                return True
            elif var_start <= pep_end <= var_end:  # partial overlap in the begining
                return True
            elif pep_start <= var_start and pep_end >= var_end:  # fully covered
                return True
        return False

    @staticmethod
    def get_altseq(ref_seq, ref_allele, var_allele, var_pos, strand, features_info, cds_info=[]):
        """
        the given sequence in the fasta file represents all exons of the transcript combined.
        for protein coding genes, the CDS is specified therefore the sequence position has to be
        calculated based on the CDS's positions
        However, for non-protein coding genes, the whole sequence is used
        """
        alt_seq = ""
        if len(cds_info) == 2:
            start_coding_index = cds_info[0] - 1  # it should be index not pos
            stop_coding_index = cds_info[1]  # get end position of the  last cds
        else:
            start_coding_index = 0
            total_len = 0
            for x in features_info:
                total_len += x[1] - x[0] + 1
            stop_coding_index = total_len  # the features are sorted by end therefroe the end pos of the last item is the last coding nc

        if strand == '-':  # ge the correct orientation, because exons are oredered based on their position
            ref_seq = ref_seq[
                      ::-1]  # in order to calculate from the first base of the first feature (sorted by genomic coordinates)
            ref_allele = ref_allele.complement()  # the reverse will be done on return
            var_allele = var_allele.complement()  # the reverse will be done on return

        ref_seq = ref_seq[
                  start_coding_index:stop_coding_index]  # just keep the coding regions (mostly effective in case of protein-coding genes)
        nc_index = 0
        if len(ref_allele) == len(var_allele) or ref_allele[0] == var_allele[0]:
            for feature in features_info:  # for every exon, cds or stop codon
                if var_pos in range(feature[0], feature[
                                                    1] + 1):  # get index of the var relative to the position of the overlapping feature in the coding region
                    var_index_in_cds = nc_index + (var_pos - feature[0])
                    # modify the coding reference sequence accoding to the var_allele
                    c = len(ref_allele)
                    alt_seq = ref_seq[0:var_index_in_cds] + var_allele + ref_seq[
                                                                         var_index_in_cds + c::]  # variant and ref strand??
                    if strand == '-':
                        return ref_seq[::-1], alt_seq[::-1]
                    else:
                        return ref_seq, alt_seq

                nc_index += (feature[1] - feature[0] + 1)

        return ref_seq, alt_seq

    @staticmethod
    def parse_gtf(gene_annotations_gtf, gtf_db_file):
        """
        Convert GTF file into a FeatureDB
        :param gene_annotations_gtf:
        :param gtf_db_file:
        :return:
        """
        try:
            gffutils.create_db(gene_annotations_gtf, gtf_db_file, merge_strategy="create_unique",
                               keep_order=True, disable_infer_transcripts=True, disable_infer_genes=True,
                               verbose=True,
                               force=False)
        except:  # already exists
            print("already exists", gtf_db_file)
            pass

        db = gffutils.FeatureDB(gtf_db_file)
        return db

    @staticmethod
    def get_features(db, feature_id, biotype_str, feature_types=['exon']):
        """
        Get chr, genomic positions, strand and biotype for feature_id
        also genomic positions for all its elements (exons/cds&start_codon)
        :param db:
        :param feature_id:
        :param biotype_str:
        :param feature_types:
        :return:
        """
        try:
            feature = db[feature_id]
        except gffutils.exceptions.FeatureNotFoundError: #remove version number from the ID
            try:
                feature = db[feature_id.split('.')[0]]
            except gffutils.exceptions.FeatureNotFoundError:
                print("""Feature {} found in fasta file but not in gtf file. Check that the fasta file and the gtf files match. 
                        A common issue is when the fasta file have chromosome patches but not the gtf""".format(feature_id))
                return None, None, None, None
        coding_features = []
        features = db.children(feature, featuretype=feature_types, order_by='end')
        for f in features:
            f_type = f.featuretype
            coding_features.append([f.start, f.end, f_type])
        return feature.chrom, feature.strand, coding_features, feature.attributes[biotype_str][0]

    @staticmethod
    def get_orfs_vcf(ref_seq: str, alt_seq: str, translation_table: int, num_orfs=1):
        """
        Translate the coding_ref and the coding_alt into ORFs
        :param ref_seq:
        :param alt_seq:
        :param translation_table:
        :param num_orfs:
        :return:
        """

        ref_orfs = []
        alt_orfs = []
        for n in range(0, num_orfs):
            ref_orfs.append(ref_seq[n::].translate(translation_table))
            alt_orfs.append(alt_seq[n::].translate(translation_table))

        return ref_orfs, alt_orfs

    @staticmethod
    def get_orfs_dna(ref_seq: str, translation_table: int, num_orfs: int, num_orfs_complement: int, to_stop: bool):
        """translate the coding_ref into ORFs"""

        ref_orfs = []
        for n in range(0, num_orfs):
            ref_orfs.append(ref_seq[n::].translate(translation_table, to_stop=to_stop))

        rev_ref_seq = ref_seq.reverse_complement()
        for n in range(0, num_orfs_complement):
            ref_orfs.append(rev_ref_seq[n::].translate(translation_table, to_stop=to_stop))

        return ref_orfs

    def dnaseq_to_proteindb(self, input_fasta):
        """
        translates DNA sequences to protein sequences
        :param input_fasta:
        :return:
        """

        seq_dict = SeqIO.index(input_fasta, "fasta")

        with open(self._proteindb_output, 'w') as prots_fn:
            for record_id in seq_dict.keys():

                ref_seq = seq_dict[
                    record_id].seq  # get the seq and desc for the record from the fasta of the gtf
                desc = str(seq_dict[record_id].description)

                key_values = {}  # extract key=value in the desc into a dict
                sep = ' '
                if '|' in desc:
                    sep = '|'
                for value in desc.split(sep):
                    if value.split('=')[0]=='cds' or value.split(':')[0]=='cds':
                        value.replace('cds', 'CDS')
                    if '=' in value:
                        key_values[value.split('=')[0]] = value.split('=')[1]
                    elif ':' in value:
                        key_values[value.split(':')[0]] = value.split(':')[1]
                    elif value.split('=')[0]=='CDS': #when only it is specified to be a CDS, it means the whole sequence to be used
                        key_values[value.split('=')[0]] = '{}-{}'.format(1, len(ref_seq))
                        
                feature_biotype = ""
                if self._biotype_str:
                    try:
                        feature_biotype = key_values[self._biotype_str]
                    except KeyError:
                        print("Biotype info was not found in the header using {} for record {} {}".format(
                            self._biotype_str, record_id, desc))
                
                # only include features that have the specified biotypes or they have CDSs info
                if 'CDS' in key_values.keys() and (
                        not self._skip_including_all_cds or 'altORFs' in self._include_biotypes):
                    pass
                elif feature_biotype == "" or (feature_biotype in self._exclude_biotypes or
                                               (
                                                       feature_biotype not in self._include_biotypes and self._include_biotypes != [
                                                   'all'])):
                    continue

                # check wether to filter on expression and if it passes
                if self._expression_str:
                    try:
                        if float(key_values[self._expression_str]) < self._expression_thresh:
                            continue
                    except KeyError:
                        print(
                            "Expression information not found in the fasta header with expression_str: {} for record {} {}".format(
                                self._expression_str, record_id, desc))
                        continue
                    except TypeError:
                        print("Expression value is not of valid type (float) at record: {} {}".format(record_id,
                                                                                                      key_values[
                                                                                                          self._expression_str]))
                        continue

                # translate the whole sequences (3 ORFs) for non CDS sequences and not take alt_ORFs for CDSs
                if 'CDS' not in key_values.keys() or ('CDS' in key_values.keys() and
                                                      ('altORFs' in self._include_biotypes or
                                                       self._include_biotypes == ['all'])):
                    ref_orfs = self.get_orfs_dna(ref_seq, self._translation_table, self._num_orfs,
                                                 self._num_orfs_complement, to_stop=False)
                    print(self._header_var_prefix)
                    self.write_output(seq_id=self._header_var_prefix+record_id, desc=desc, seqs=ref_orfs, prots_fn=prots_fn)

                # also allow for direct translation of the CDS, when the cds info exists in the fasta header skip_including_all_cds is false
                if 'CDS' in key_values.keys() and not self._skip_including_all_cds:
                    try:
                        cds_info = [int(x) for x in key_values['CDS'].split('-')]
                        ref_seq = ref_seq[cds_info[0] - 1:cds_info[1]]
                        ref_orfs = self.get_orfs_dna(ref_seq, self._translation_table, 1, 0, to_stop=True)
                        self.write_output(seq_id=record_id, desc=desc, seqs=ref_orfs, prots_fn=prots_fn)
                    except (ValueError, IndexError, KeyError):
                        print("Could not extra cds position from fasta header for: ", record_id, desc)

        return self._proteindb_output

    @staticmethod
    def get_key(fasta_header):
        return fasta_header.split('|')[0].split(' ')[0]
    
    def vcf_to_proteindb(self, vcf_file, input_fasta, gene_annotations_gtf):
        """
        Generate peps for variants by modifying sequences of affected transcripts (VCF - VEP annotated).
        It only considers variants within potential coding regions of the transcript
        (CDSs & stop codons for protein-coding genes, exons for non-protein coding genes).
        :param vcf_file:
        :param input_fasta:
        :param gene_annotations_gtf:
        :return:
        """
        
        db = self.parse_gtf(gene_annotations_gtf, gene_annotations_gtf.replace('.gtf', '.db'))

        transcripts_dict = SeqIO.index(input_fasta, "fasta", key_function=self.get_key)
        # handle cases where the transript has version in the GTF but not in the VCF
        transcript_id_mapping = {k.split('.')[0]: k for k in transcripts_dict.keys()}
        with open(self._proteindb_output, 'w') as prots_fn:
            vcf_reader = vcf.Reader(open(vcf_file, 'r'))
            
            for record in vcf_reader:
                if not self._ignore_filters:
                    if record.FILTER:  # if not PASS: None and empty means PASS
                        if not (set(record.FILTER[0].split(',')) <= set(self._accepted_filters)):
                            continue

                # only process variants above a given allele frequency threshold if the AF string is not empty
                if self._af_field:
                    # get AF from the INFO field
                    try:
                        af = float(record.INFO[self._af_field])
                    except TypeError:
                        af = float(record.INFO[self._af_field][0])
                    except KeyError:
                        continue

                    # check if the AF passed the threshold
                    if af < self._af_threshold:
                        continue

                trans_table = self._translation_table
                consequences = []
                if str(record.CHROM).lstrip('chr').upper() in ['M', 'MT']:
                    trans_table = self._mito_translation_table

                processed_transcript_allele = []
                
                for transcript_record in record.INFO[self._annotation_field_name]:
                    transcript_info = transcript_record.split('|')
                    try:
                        consequence = transcript_info[self._consequence_index]
                    except IndexError:
                        print("Give a valid index for the consequence in the INFO field for: ",
                              transcript_record)
                        continue
                    consequences.append(consequence)
                    try:
                        transcript_id = transcript_info[self._transcript_index]
                    except IndexError:
                        print("Give a valid index for the Transcript ID in the INFO field for: ",
                              transcript_record)
                        continue
                    if transcript_id == "":
                        continue

                    try:
                        transcript_id_v = transcript_id_mapping[transcript_id]
                    except KeyError:
                        transcript_id_v = transcript_id

                    try:
                        row = transcripts_dict[transcript_id_v]
                        ref_seq = row.seq  # get the seq and desc for the transcript from the fasta of the gtf
                        desc = str(row.description)
                    except KeyError:
                        print("Transcript {} not found in fasta of the GTF file {}".format(
                            transcript_id_v, record))
                        continue

                    feature_types = ['exon']
                    # check if cds info exists in the fasta header otherwise translate all exons
                    cds_info = []
                    num_orfs = 3
                    if 'CDS=' in desc:
                        try:
                            cds_info = [int(x) for x in desc.split(' ')[1].split('=')[1].split('-')]
                            feature_types = ['CDS', 'stop_codon']
                            num_orfs = 1
                        except (ValueError, IndexError):
                            print("Could not extra cds position from fasta header for: ", desc)
                            pass
                    chrom, strand, features_info, feature_biotype = self.get_features(db, transcript_id_v,
                                                                                      self._biotype_str,
                                                                                      feature_types)
                    if chrom == None: #the record info was not found
                        continue
                    # skip transcripts with unwanted consequences
                    if (consequence in self._exclude_consequences or
                            (consequence not in self._include_consequences and
                             self._include_consequences != ['all'])):
                        continue

                    # only include features that have the specified biotypes or they have CDSs info
                    if 'CDS' in feature_types and not self._skip_including_all_cds:
                        pass
                    elif (feature_biotype in self._exclude_biotypes or
                          (feature_biotype not in self._include_biotypes and
                           self._include_biotypes != ['all'])):
                        continue
                    for alt in record.ALT:  # in cases of multiple alternative alleles consider all
                        if transcript_id + str(record.REF) + str(
                                alt) in processed_transcript_allele:  # because VEP reports affected transcripts per alt allele
                            continue

                        processed_transcript_allele.append(transcript_id + str(record.REF) + str(alt))
                        "for non-CDSs, only consider the exon that actually overlaps the variant"
                        if (chrom.lstrip("chr") == str(record.CHROM).lstrip("chr") and
                                self.check_overlap(record.POS, record.POS + len(alt), features_info)):
                            coding_ref_seq, coding_alt_seq = self.get_altseq(ref_seq, Seq(str(record.REF)),
                                                                             Seq(str(alt)), int(record.POS), strand,
                                                                             features_info, cds_info)
                            if coding_alt_seq != "":
                                ref_orfs, alt_orfs = self.get_orfs_vcf(coding_ref_seq, coding_alt_seq, trans_table,
                                                                       num_orfs)
                                record_id = ""
                                if record.ID:
                                    record_id = '_' + str(record.ID)
                                self.write_output(seq_id='_'.join([self._header_var_prefix + str(record_id),
                                                                   '.'.join([str(record.CHROM), str(record.POS),
                                                                             str(record.REF), str(alt)]),
                                                                   transcript_id_v]),
                                                  desc=feature_biotype + ":" + consequence,
                                                  seqs=alt_orfs,
                                                  prots_fn=prots_fn)

                                if self._report_reference_seq:
                                    self.write_output(seq_id=transcript_id_v,
                                                      desc=feature_biotype,
                                                      seqs=ref_orfs,
                                                      prots_fn=prots_fn)

        return self._proteindb_output

    def write_output(self, seq_id, desc, seqs, prots_fn):
        """write the orfs to the output file"""
        write_i = False
        if len(seqs) > 1:  # only add _num when multiple ORFs are generated (e.g in 3 ORF)
            write_i = True

        for i, orf in enumerate(seqs):
            if write_i:  # only add _num when multiple ORFs are generated (e.g in 3 ORF)
                prots_fn.write('>{} {}\n{}\n'.format(seq_id + "_" + str(i + 1), desc, orf))
            else:
                prots_fn.write('>{} {}\n{}\n'.format(seq_id, desc, orf))


if __name__ == '__main__':
    print("ERROR: This script is part of a pipeline collection and it is not meant to be run in stand alone mode")



import click

from pypgatk.commands.utils import print_help
from pypgatk.ensembl.ensembl import EnsemblDataService


@click.command("dnaseq-to-proteindb", short_help="Generate peptides based on DNA sequences")
@click.option('--config_file', '-c', help='Configuration to perform conversion between ENSEMBL Files',
              default='config/ensembl_config.yaml')
@click.option('--input_fasta', help='Path to sequences fasta')
@click.option('--translation_table', default=1, type=int, help='Translation Table (default 1)')
@click.option('--num_orfs', default=3, type=int, help='Number of ORFs (default 0)')
@click.option('--num_orfs_complement', default=0, type=int,
              help='Number of ORFs from the reverse side (default 0)')
@click.option('--output_proteindb', default="peptide-database.fa", help="Output file name, exits if already exists")
@click.option('-p', '--var_prefix', default="", help="String to add before the variant peptides")

@click.option('--skip_including_all_cds',
              help="By default any transcript that has a defined CDS will be translated, this option disables this features instead it only depends on the biotypes",
              is_flag=True)
@click.option('--include_biotypes', default='', help="Include Biotypes")
@click.option('--exclude_biotypes', default='', help="Exclude Biotypes")
@click.option('--biotype_str', default='transcript_biotype', type=str,
              help='String used to identify gene/transcript biotype in the gtf file.')
@click.option('--expression_str', default="", type=str,
              help='String to be used for extracting expression value (TPM, FPKM, etc).')
@click.option('--expression_thresh', default=5.0, type=float,
              help='Threshold used to filter transcripts based on their expression values')
@click.pass_context
def dnaseq_to_proteindb(ctx, config_file, input_fasta, translation_table, num_orfs, num_orfs_complement,
                        output_proteindb, var_prefix,
                        skip_including_all_cds, include_biotypes, exclude_biotypes, biotype_str, expression_str,
                        expression_thresh):
    if input_fasta is None:
        print_help()

    pipeline_arguments = {}
    pipeline_arguments[EnsemblDataService.TRANSLATION_TABLE] = translation_table
    pipeline_arguments[EnsemblDataService.PROTEIN_DB_OUTPUT] = output_proteindb
    pipeline_arguments[EnsemblDataService.HEADER_VAR_PREFIX] = var_prefix
    pipeline_arguments[EnsemblDataService.EXCLUDE_BIOTYPES] = exclude_biotypes
    pipeline_arguments[EnsemblDataService.SKIP_INCLUDING_ALL_CDS] = skip_including_all_cds
    pipeline_arguments[EnsemblDataService.INCLUDE_BIOTYPES] = include_biotypes
    pipeline_arguments[EnsemblDataService.BIOTYPE_STR] = biotype_str
    pipeline_arguments[EnsemblDataService.NUM_ORFS] = num_orfs
    pipeline_arguments[EnsemblDataService.NUM_ORFS_COMPLEMENT] = num_orfs_complement
    pipeline_arguments[EnsemblDataService.EXPRESSION_STR] = expression_str
    pipeline_arguments[EnsemblDataService.EXPRESSION_THRESH] = expression_thresh

    ensembl_data_service = EnsemblDataService(config_file, pipeline_arguments)
    ensembl_data_service.dnaseq_to_proteindb(input_fasta)




import click

from pypgatk.commands.utils import print_help
from pypgatk.ensembl.ensembl import EnsemblDataService


@click.command('threeframe-translation', short_help="Command to perform 3'frame translation")
@click.option('--config_file',
              '-c',
              help='Configuration to perform conversion between ENSEMBL Files',
              default='config/ensembl_config.yaml')
@click.option('-in', '--input_fasta', help='input_fasta file to perform the translation')
@click.option('-t', '--translation_table', help='Translation table default value 1', default='1')
@click.option('-out', '--output', help='Output File', default="peptide-database.fa")
@click.pass_context
def threeframe_translation(ctx, config_file, input_fasta, translation_table, output):
    if input_fasta is None:
        print_help()

    pipeline_arguments = {EnsemblDataService.TRANSLATION_TABLE: translation_table,
                          EnsemblDataService.PROTEIN_DB_OUTPUT: output}

    ensembl_data_service = EnsemblDataService(config_file, pipeline_arguments)
    ensembl_data_service.three_frame_translation(input_fasta)



#
# DecoyPYrat - Fast Hybrid Decoy Sequence Database Creation for Proteomic Mass Spectromtery Analyses
#
# MIT License
#
# Copyright (c) 2016 James Christopher Wright - Wellcome Trust Sanger Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import random
import os
from Bio import SeqIO

from pypgatk.toolbox.general import ParameterConfiguration


class ProteinDBService(ParameterConfiguration):

    CONFIG_KEY_PROTEINDB_DECOY = 'proteindb_decoy'
    CONFIG_PROTEINDB_OUTPUT = 'output'
    CONFIG_INPUT_FILE = 'input'
    CONFIG_CLEAVAGE_SITES = 'cleavage_sites'
    CONFIG_CLEAVAGE_POSITION = 'cleavage_position'
    CONFIG_ANTI_CLEAVAGE_SITES = 'anti_cleavage_sites'
    CONFIG_PEPTIDE_LENGTH = 'min_peptide_length'
    CONFIG_MAX_ITERATIONS = 'max_iterations'
    CONFIG_DO_NOT_SUFFLE = 'do_not_shuffle'
    CONFIG_DO_NOT_SWITCH = 'do_not_switch'
    CONFIG_DECOY_PREFIX = 'decoy_prefix'
    CONFIG_TEMP_FILE = 'temp_file'
    CONFIG_NO_ISOBARIC = 'no_isobaric'
    CONFIG_MEMORY_SAVE = 'memory_save'

    def __init__(self, config_file, pipeline_arguments):
        """
        Init the class with the specific parameters.
        :param config_file configuration file
        :param pipeline_arguments pipelines arguments
        """

        super(ProteinDBService, self).__init__(self.CONFIG_KEY_PROTEINDB_DECOY, config_file,
                                               pipeline_arguments)

        self._temp_file = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][self.CONFIG_TEMP_FILE]
        if self.CONFIG_TEMP_FILE in self.get_pipeline_parameters():
            self._temp_file = self.get_pipeline_parameters()[self.CONFIG_TEMP_FILE]

        self._input_fasta = self.get_pipeline_parameters()[self.CONFIG_INPUT_FILE]

        self._isobaric = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][self.CONFIG_NO_ISOBARIC]
        if self.CONFIG_NO_ISOBARIC in self.get_pipeline_parameters():
            self._isobaric = self.get_pipeline_parameters()[self.CONFIG_NO_ISOBARIC]

        self._memory_save = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][self.CONFIG_MEMORY_SAVE]
        if self.CONFIG_MEMORY_SAVE in self.get_pipeline_parameters():
            self._memory_save = self.get_pipeline_parameters()[self.CONFIG_MEMORY_SAVE]

        self._cleavage_sites = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][
            self.CONFIG_CLEAVAGE_SITES]
        if self.CONFIG_CLEAVAGE_SITES in self.get_pipeline_parameters():
            self._cleavage_sites = self.get_pipeline_parameters()[self.CONFIG_CLEAVAGE_SITES]

        self._decoy_prefix = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][
            self.CONFIG_DECOY_PREFIX]
        if self.CONFIG_DECOY_PREFIX in self.get_pipeline_parameters():
            self._decoy_prefix = self.get_pipeline_parameters()[self.CONFIG_DECOY_PREFIX]

        self._cleavage_position = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][
            self.CONFIG_CLEAVAGE_POSITION]
        if self.CONFIG_CLEAVAGE_POSITION in self.get_pipeline_parameters():
            self._cleavage_position = self.get_pipeline_parameters()[self.CONFIG_CLEAVAGE_POSITION]

        self._anti_cleavage_sites = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][
            self.CONFIG_ANTI_CLEAVAGE_SITES]
        if self.CONFIG_ANTI_CLEAVAGE_SITES in self.get_pipeline_parameters():
            self._anti_cleavage_sites = self.get_pipeline_parameters()[self.CONFIG_ANTI_CLEAVAGE_SITES]

        self._peptide_length = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][
            self.CONFIG_PEPTIDE_LENGTH]
        if self.CONFIG_PEPTIDE_LENGTH in self.get_pipeline_parameters():
            self._peptide_length = self.get_pipeline_parameters()[self.CONFIG_PEPTIDE_LENGTH]

        self._max_iterations = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][
            self.CONFIG_MAX_ITERATIONS]
        if self.CONFIG_MAX_ITERATIONS in self.get_pipeline_parameters():
            self._max_iterations = self.get_pipeline_parameters()[self.CONFIG_MAX_ITERATIONS]

        self._no_switch = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][
            self.CONFIG_DO_NOT_SWITCH]
        if self.CONFIG_DO_NOT_SWITCH in self.get_pipeline_parameters():
            self._no_switch = self.get_pipeline_parameters()[self.CONFIG_DO_NOT_SWITCH]

        self._output_file = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][
            self.CONFIG_PROTEINDB_OUTPUT]
        if self.CONFIG_PROTEINDB_OUTPUT in self.get_pipeline_parameters():
            self._output_file = self.get_pipeline_parameters()[self.CONFIG_PROTEINDB_OUTPUT]

        self._no_suffle = self.get_default_parameters()[self.CONFIG_KEY_PROTEINDB_DECOY][
            self.CONFIG_DO_NOT_SUFFLE]
        if self.CONFIG_DO_NOT_SUFFLE in self.get_pipeline_parameters():
            self._no_suffle = self.get_pipeline_parameters()[self.CONFIG_DO_NOT_SUFFLE]

    @staticmethod
    def digest(protein, sites, pos, no, min_peptide_length):
        """
        Return a list of cleaved peptides with minimum length in protein sequence.
        :param protein = sequence
        :param sites = string of amino acid cleavage sites
        :param pos = n or c for n-terminal or c-terminal cleavage
        :param no = amino acids following site that would prevent cleavage ie proline
        :param min_peptide_length = minimum length of peptides returned
        """

        # for each possible cleavage site insert a comma with before or after depending on pos
        for s in sites:
            r = s + ','
            if pos == 'n':
                r = ',' + s
            protein = protein.replace(s, r)

        # for each possible cleavage and all none cleavage remove comma
        for s in sites:
            for n in no:
                a = s + ',' + n
                if pos == 'n':
                    a = ',' + s + n
                r = s + n
                protein = protein.replace(a, r)

        # filter peptides into list by minimum size
        return list(filter(lambda x: len(x) >= min_peptide_length, (protein.split(','))))

    @staticmethod
    def revswitch(protein, noswitch, sites):
        """
        Return a reversed protein sequence with cleavage residues switched with preceding residue
        """
        # reverse protein sequence with a reverse splice convert to list
        revseq = list(protein[::-1])

        if not noswitch:

            # loop sequence list
            for i, c in enumerate(revseq):
                # if value is cleavage site switch with previous amino acid
                for s in sites:
                    if c == s:
                        aa = revseq[i - 1]
                        revseq[i - 1] = revseq[i]
                        revseq[i] = aa

        # return reversed with/without switched proteins as string
        return ''.join(revseq)

    @staticmethod
    def shuffle(peptide):
        """
        shuffle peptide without moving c-terminal amino acid cleavage site
        """

        # extract terminal aa
        s = peptide[-1]
        # convert peptide to list (remove K/R) and shuffle the list
        l = list(peptide[:-1])
        random.shuffle(l)
        # return new peptide
        return ''.join(l) + s

    def decoy_database(self):
        """
        Create a decoy database from a proteomics database
        :return:
        """

        # Create empty sets to add all target and decoy peptides
        upeps = set()
        dpeps = set()

        # Counter for number of decoy sequences
        dcount = 0

        # empty protein sequence
        seq = ''

        # open temporary decoy FASTA file
        outfa = open(self._temp_file, 'w')

        # Open FASTA file using first cmd line argument
        fasta = SeqIO.parse(self._input_fasta, 'fasta')
        # loop each seq in the file
        for record in fasta:
                seq = str(record.seq)
                dcount+=1	
                #make sequence isobaric (check args for switch off)
                if self._isobaric == False:
                        seq = seq.replace('I', 'L')

                #digest sequence add peptides to set
                upeps.update(ProteinDBService.digest(seq, self._cleavage_sites, self._cleavage_position,
                                                         self._anti_cleavage_sites, self._peptide_length))

                #reverse and switch protein sequence
                decoyseq = ProteinDBService.revswitch(seq, self._no_switch, self._cleavage_sites)

                #do not store decoy peptide set in reduced memory mode
                if self._memory_save == False:
                        #update decoy peptide set
                        dpeps.update( ProteinDBService.digest(decoyseq, self._cleavage_sites, self._cleavage_position,
                                                             self._anti_cleavage_sites, self._peptide_length))

                #write decoy protein accession and sequence to file
                outfa.write('>' + self._decoy_prefix + record.id + '\n')
                outfa.write(decoyseq + '\n')

        # Close files
        fasta.close()
        outfa.close()

        # Summarise the numbers of target and decoy peptides and their intersection
        nonDecoys = set()
        print("proteins:" + str(dcount))
        print("target peptides:" + str(len(upeps)))

        # Reloop decoy file in reduced memory mode to store only intersecting decoys
        if self._memory_save:
            # open temp decoys
            with open(self._temp_file, "rt") as fin:
                for line in fin:
                    # if line is not accession
                    if line[0] != '>':
                        # digest protein
                        for p in ProteinDBService.digest(line.rstrip(), self._cleavage_sites, self._cleavage_position,
                                                         self._anti_cleavage_sites, self._peptide_length):
                            # check if in target peptides if true then add to nonDecoys
                            if p in upeps:
                                nonDecoys.add(p)
            fin.close()
            print("decoy peptides: !Memory Saving Made!")
        else:
            # can only report total number in normal memory mode
            print("decoy peptides:" + str(len(dpeps)))
            # find intersecting peptides
            nonDecoys = upeps.intersection(dpeps)

        print("#intersection:" + str(len(nonDecoys)))

        # if there are decoy peptides that are in the target peptide set
        if len(nonDecoys) > 0 and self._no_suffle == False:

            # create empty dictionary with bad decoys as keys
            dAlternative = dict.fromkeys(nonDecoys, '')
            noAlternative = list()

            # loop bad decoys / dictionary keys
            for dPep in dAlternative:
                i = 0
                aPep = dPep

                # shuffle until aPep is not in target set (maximum of 10 iterations)
                while aPep in upeps and i < self._max_iterations:

                    # increment iteration counter
                    i += 1

                    # shuffle peptide
                    aPep = ProteinDBService.shuffle(dPep)

                    # check if shuffling has an effect if not end iterations
                    if aPep == dPep:
                        i = self._max_iterations

                # update dictionary with alternative shuffled peptide
                dAlternative[dPep] = aPep

                # warn if peptide has no suitable alternative, add to removal list
                if i == self._max_iterations:
                    noAlternative.append(dPep)

            print(str(len(noAlternative)) + ' have no alternative peptide')
            # remove peptides with no alternative
            for p in noAlternative:
                del dAlternative[p]

            # Free up memory by clearing large sets of peptides
            upeps.clear()
            dpeps.clear()
            # open second decoy file
            with open(self._output_file, "wt") as fout:
                # open original decoy file
                with open(self._temp_file, "rt") as fin:
                    # loop each line of original decoy fasta
                    for line in fin:
                        # if line is not accession replace peptides in dictionary with alternatives
                        if line[0] != '>':
                            # digest decoy sequence
                            for p in ProteinDBService.digest(line.rstrip(), self._cleavage_sites,
                                                             self._cleavage_position, self._anti_cleavage_sites,
                                                             self._peptide_length):
                                # store decoy peptide for final count
                                dpeps.add(p)

                                # if decoy peptide is in dictionary replace with alternative
                                if p in dAlternative:
                                    line = line.replace(p, dAlternative[p])

                        fout.write(line)
                fin.close()
            fout.close()

            # delete temporary file
            os.remove(self._temp_file)
        else:
            os.rename(self._temp_file, self._output_file)

        print("final decoy peptides:" + str(len(dpeps)))



import csv
from concurrent.futures import as_completed
from concurrent.futures.thread import ThreadPoolExecutor

from requests import get

from pypgatk.toolbox.exceptions import AppException
from pypgatk.toolbox.general import ParameterConfiguration, check_create_folders, download_file, clear_cache


class CbioPortalDownloadService(ParameterConfiguration):
    CONFIG_KEY_DATA_DOWNLOADER = 'cbioportal_data_downloader'
    CONFIG_KEY_CBIOPORTAL_DOWNLOAD_URL = 'cbioportal_download_url'
    CONFIG_OUTPUT_DIRECTORY = 'output_directory'
    CONFIG_CBIOPORTAL_API = 'cbioportal_api'
    CONFIG_CBIOPORTAL_API_SERVER = 'base_url'
    CONFIG_CBIOPORTAL_API_CANCER_STUDIES = "cancer_studies"
    CONFIG_LIST_STUDIES = "list_studies"
    CONFIG_MULTITHREADING = "multithreading"

    def __init__(self, config_file, pipeline_arguments):
        """
        Init the class with the specific parameters.
        :param config_file configuration file
        :param pipeline_arguments pipelines arguments
        """
        super(CbioPortalDownloadService, self).__init__(self.CONFIG_KEY_DATA_DOWNLOADER, config_file,
                                                        pipeline_arguments)

        self._cbioportal_studies = []
        if self.CONFIG_OUTPUT_DIRECTORY in self.get_pipeline_parameters():
            self._local_path_cbioportal = self.get_pipeline_parameters()[self.CONFIG_OUTPUT_DIRECTORY]
        else:
            self._local_path_cbioportal = self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][
                self.CONFIG_OUTPUT_DIRECTORY]

        self._list_studies = self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_LIST_STUDIES]
        if self.CONFIG_LIST_STUDIES in self.get_pipeline_parameters():
            self._list_studies = self.get_pipeline_parameters()[self.CONFIG_LIST_STUDIES]

        self._multithreading = self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][
            self.CONFIG_MULTITHREADING]
        if self.CONFIG_MULTITHREADING in self.get_pipeline_parameters():
            self._multithreading = self.get_pipeline_parameters()[self.CONFIG_MULTITHREADING]

        self.prepare_local_cbioportal_repository()

    def prepare_local_cbioportal_repository(self):
        self.get_logger().debug("Preparing local cbioportal repository, root folder - '{}'".format(
            self.get_local_path_root_cbioportal_repo()))
        check_create_folders([self.get_local_path_root_cbioportal_repo()])
        self.get_logger().debug(
            "Local path for cbioportal Release - '{}'".format(self.get_local_path_root_cbioportal_repo()))

    def get_local_path_root_cbioportal_repo(self):
        return self._local_path_cbioportal

    def get_cancer_studies(self):
        """
        This method will print the list of all cancer studies for the user.
        :return:
        """
        server = self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_CBIOPORTAL_API][
            self.CONFIG_CBIOPORTAL_API_SERVER]
        endpoint = self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_CBIOPORTAL_API][
            self.CONFIG_CBIOPORTAL_API_CANCER_STUDIES]
        self._cbioportal_studies = get(server + "?" + endpoint).text
        return self._cbioportal_studies

    def download_study(self, download_study):
        """
        This function will download a study from cBioPortal using the study ID
        :param download_study: Study to be download, if the study is empty or None, all the studies will be
        downloaded.
        :return: None
        """

        clear_cache()

        if self._cbioportal_studies is None or len(self._cbioportal_studies) == 0:
            self.get_cancer_studies()

        if 'all' not in download_study:
            if not self.check_study_identifier(download_study):
                msg = "The following study accession '{}' is not present in cBioPortal Studies".format(download_study)
                self.get_logger().debug(msg)
                raise AppException(msg)
            else:
                self.download_one_study(download_study)
        else:
            csv_reader = csv.reader(self._cbioportal_studies.splitlines(), delimiter="\t")
            line_count = 0
            if self._multithreading:
                processes = []
                with ThreadPoolExecutor(max_workers=10, thread_name_prefix='Thread-Download') as executor:
                    for row in csv_reader:
                        if line_count != 0:
                            processes.append(executor.submit(self.download_one_study, row[0]))
                        line_count = line_count + 1
                for task in as_completed(processes):
                    print(task.result())
            else:
                for row in csv_reader:
                    if line_count != 0:
                        self.download_one_study(row[0])
                    line_count = line_count + 1

    def download_one_study(self, download_study):
        file_name = '{}.tar.gz'.format(download_study)
        file_url = '{}/{}'.format(
            self.get_default_parameters()[self.CONFIG_KEY_DATA_DOWNLOADER][self.CONFIG_KEY_CBIOPORTAL_DOWNLOAD_URL],
            file_name)
        file_name = download_file(file_url, self.get_local_path_root_cbioportal_repo() + '/' + file_name, self.get_logger())
        if file_name is not None:
            msg = "The following study '{}' has been downloaded. ".format(download_study)
        else:
            msg = "The following study '{}' hasn't been downloaded. ".format(download_study)
        self.get_logger().debug(msg)
        return file_name

    def check_study_identifier(self, download_study):
        return download_study in self._cbioportal_studies



import logging

import click

from pypgatk.proteomics.db.decoy_pyrat import ProteinDBService
from pypgatk.toolbox.exceptions import AppConfigException


@click.command('generate-decoy', short_help='Create decoy protein sequences. Each protein '
                                             'is reversed and the cleavage sites switched with preceding amino acid. Peptides are checked for existence in target sequences if found'
                                             'the tool will attempt to shuffle them. James.Wright@sanger.ac.uk 2015')
@click.option('--config_file',         '-c', help='Configuration file for the protein database decoy generation', default='config/protein_decoy.yaml')
@click.option('--output',              '-o', help='Output file for decoy database', default="protein-decoy.fa")
@click.option('--input',               '-i', help='FASTA file of target proteins sequences for which to create decoys (*.fasta|*.fa)')
@click.option('--cleavage_sites',      '-s', default='KR', help='A list of amino acids at which to cleave during digestion. Default = KR')
@click.option('--anti_cleavage_sites', '-a', help='A list of amino acids at which not to cleave if following cleavage site ie. Proline. Default = none')
@click.option('--cleavage_position',   '-p', default='c', help='Set cleavage to be c or n terminal of specified cleavage sites. Options [c, n], Default = c')
@click.option('--min_peptide_length',  '-l', type=int, help='Set minimum length of peptides to compare between target and decoy. Default = 5')
@click.option('--max_iterations',      '-n', type=int, help='Set maximum number of times to shuffle a peptide to make it non-target before failing. Default=100')
@click.option('--do_not_shuffle',      '-x', help='Turn OFF shuffling of decoy peptides that are in the target database. Default=false')
@click.option('--do_not_switch',       '-w', help='Turn OFF switching of cleavage site with preceding amino acid. Default=false')
@click.option('--decoy_prefix',        '-d', help='Set accession prefix for decoy proteins in output. Default=DECOY_')
@click.option('--temp_file',           '-t', help='Set temporary file to write decoys prior to shuffling. Default=protein-decoy.fa')
@click.option('--no_isobaric',         '-b', help='Do not make decoy peptides isobaric. Default=false')
@click.option('--memory_save',         '-m', help='Slower but uses less memory (does not store decoy peptide list). Default=false')
@click.pass_context
def generate_database(ctx, config_file, output, input, cleavage_sites, anti_cleavage_sites, cleavage_position, min_peptide_length,
                   max_iterations, do_not_shuffle, do_not_switch, decoy_prefix, temp_file, no_isobaric, memory_save):

    if config_file is None:
        msg = "The config file for the pipeline is missing, please provide one "
        logging.error(msg)
        raise AppConfigException(msg)

    pipeline_arguments = {}

    if output is not None:
        pipeline_arguments[ProteinDBService.CONFIG_PROTEINDB_OUTPUT] = output

    if input is not None:
        pipeline_arguments[ProteinDBService.CONFIG_INPUT_FILE] = input

    if cleavage_position is not None:
        pipeline_arguments[ProteinDBService.CONFIG_CLEAVAGE_SITES] = cleavage_sites

    if cleavage_position is not None:
        pipeline_arguments[ProteinDBService.CONFIG_CLEAVAGE_POSITION] = cleavage_position

    if anti_cleavage_sites is not None:
        pipeline_arguments[ProteinDBService.CONFIG_ANTI_CLEAVAGE_SITES] = anti_cleavage_sites

    if min_peptide_length is not None:
        pipeline_arguments[ProteinDBService.CONFIG_PEPTIDE_LENGTH] = min_peptide_length

    if max_iterations is not None:
        pipeline_arguments[ProteinDBService.CONFIG_MAX_ITERATIONS] = max_iterations

    if do_not_shuffle is not None:
        pipeline_arguments[ProteinDBService.CONFIG_DO_NOT_SUFFLE] = do_not_shuffle

    if do_not_switch is not None:
        pipeline_arguments[ProteinDBService.CONFIG_DO_NOT_SWITCH] = do_not_switch

    if decoy_prefix is not None:
        pipeline_arguments[ProteinDBService.CONFIG_DECOY_PREFIX] = decoy_prefix

    if temp_file is not None:
        pipeline_arguments[ProteinDBService.CONFIG_TEMP_FILE] = temp_file

    if no_isobaric is not None:
        pipeline_arguments[ProteinDBService.CONFIG_NO_ISOBARIC] = no_isobaric

    if memory_save is not None:
        pipeline_arguments[ProteinDBService.CONFIG_MEMORY_SAVE] = memory_save

    proteindb_decoy = ProteinDBService(config_file, pipeline_arguments)
    proteindb_decoy.decoy_database()



from click.testing import CliRunner

from pypgatk.pypgatk_cli import cli


def vcf_to_proteindb():
    """
    Test the default behaviour of the vcf-to-proteindb tool
    :return:
    """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['vcf-to-proteindb', '--config_file', 'config/ensembl_config.yaml',
                            '--vep_annotated_vcf', 'testdata/test.vcf',
                            '--input_fasta', 'testdata/test.fa',
                            '--gene_annotations_gtf', 'testdata/test.gtf',
                            '--var_prefix', 'ensvar',
                            '--af_field', 'MAF',
                            '--output_proteindb', 'testdata/proteindb_from_ENSEMBL_VCF.fa'])
    assert result.exit_code == 0


def vcf_gnomad_to_proteindb():
    """
    Test the default behaviour of the vcf-to-proteindb tool
    :return:
    """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['vcf-to-proteindb', '--config_file', 'config/ensembl_config.yaml',
                            '--vep_annotated_vcf', 'testdata/test_gnomad.vcf',
                            '--input_fasta', 'testdata/test_gencode.fa',
                            '--gene_annotations_gtf', 'testdata/test_gencode.gtf',
                            '--output_proteindb', 'testdata/proteindb_from_gnomad_VCF.fa',
                            '--af_field', 'controls_AF',
                            '--var_prefix', 'gnomvar',
                            '--transcript_index', 6,
                            '--biotype_str', 'transcript_type',
                            '--annotation_field_name', 'vep'])
    assert result.exit_code == 0


def dnaseq_to_proteindb():
    """
    Test the default behaviour (translate CDSs) of the dnaseq-to-proteindb tool
    :return:
    """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['dnaseq-to-proteindb', '--config_file', 'config/ensembl_config.yaml',
                            '--input_fasta', 'testdata/test.fa',
                            '--output_proteindb', 'testdata/proteindb_from_CDSs_DNAseq.fa'])
    assert result.exit_code == 0

def dnaseq_ncrnas_to_proteindb():
    """
    Test generation of proteinDB from short noncoding RNAs using dnaseq-to-proteindb tool
    :return:
    """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['dnaseq-to-proteindb', '--config_file', 'config/ensembl_config.yaml',
                            '--input_fasta', 'testdata/test.fa',
                            '--output_proteindb', 'testdata/proteindb_from_ncRNAs_DNAseq.fa',
                            '--var_prefix', 'ncRNA_',
                            '--include_biotypes',
                            'lncRNA,retained_intron,Mt_rRNA,Mt_tRNA,miRNA,misc_RNA,rRNA,ribozyme,sRNA,scRNA,scaRNA,snRNA,snoRNA,vaultRNA',
                            '--skip_including_all_cds'])
    assert result.exit_code == 0

def dnaseq_lncrnas_to_proteindb():
    """
    Test generation of proteinDB from long noncoding RNAs using dnaseq-to-proteindb tool
    :return:
    """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['dnaseq-to-proteindb', '--config_file', 'config/ensembl_config.yaml',
                            '--input_fasta', 'testdata/test.fa',
                            '--output_proteindb', 'testdata/proteindb_from_lncRNAs_DNAseq.fa',
                            '--var_prefix', 'lncRNA_',
                            '--include_biotypes',
                            'lncRNA,retained_intron',
                            '--skip_including_all_cds'])
    assert result.exit_code == 0


def dnaseq_sncrnas_to_proteindb():
    """
    Test generation of proteinDB from short noncoding RNAs using dnaseq-to-proteindb tool
    :return:
    """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['dnaseq-to-proteindb', '--config_file', 'config/ensembl_config.yaml',
                            '--input_fasta', 'testdata/test.fa',
                            '--output_proteindb', 'testdata/proteindb_from_sncRNAs_DNAseq.fa',
                            '--var_prefix', 'sncRNA_',
                            '--include_biotypes',
                            'Mt_rRNA,Mt_tRNA,miRNA,misc_RNA,rRNA,ribozyme,sRNA,scRNA,scaRNA,snRNA,snoRNA,vaultRNA',
                            '--skip_including_all_cds'])
    assert result.exit_code == 0


def dnaseq_pseudogenes_to_proteindb():
    """
    Test generation of proteinDB from pseudogenes using dnaseq-to-proteindb tool
    :return:
    """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['dnaseq-to-proteindb', '--config_file', 'config/ensembl_config.yaml',
                            '--input_fasta', 'testdata/test.fa',
                            '--output_proteindb', 'testdata/proteindb_from_pseudogenes_DNAseq.fa',
                            '--var_prefix', 'pseudo_',
                            '--include_biotypes',
                            'disrupted_domain, IGC_pseudogene, IGJ_pseudogene, IG_pseudogene, IGV_pseudogene, processed_pseudogene, transcribed_processed_pseudogene, transcribed_unitary_pseudogene, transcribed_unprocessed_pseudogene, translated_processed_pseudogene, TRJ_pseudogene, unprocessed_pseudogene',
                            '--skip_including_all_cds'])
    assert result.exit_code == 0


def dnaseq_altorfs_to_proteindb():
    """
    Test generation proteinDB from altORFs using dnaseq-to-proteindb tool
    :return:
    """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['dnaseq-to-proteindb', '--config_file', 'config/ensembl_config.yaml',
                            '--input_fasta', 'testdata/test.fa',
                            '--output_proteindb', 'testdata/proteindb_from_altORFs_DNAseq.fa',
                            '--var_prefix', 'altorf_',
                            '--include_biotypes', 'altORFs', '--skip_including_all_cds'])
    assert result.exit_code == 0


def cbioportal_to_proteindb():
    """
    Test generation proteinDB from cBioportal mutations using cbioportal-to-proteindb tool
    :return:
    """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['cbioportal-to-proteindb', '--config_file', 'config/cbioportal_config.yaml',
                            '--input_mutation', 'testdata/test_cbioportal_data_mutations_mskcc.txt',
                            '--input_cds', 'testdata/test_cbioportal_genes.fa',
                            '--output_db', 'testdata/test_cbioportal_data_mutations_mskcc_proteindb.fa',
                            '--clinical_sample_file', 'testdata/test_cbioportal_data_clinical_sample.txt',
                            '--split_by_tissue_type', '--tissue_type', 'all'])
    assert result.exit_code == 0


def cosmic_to_proteindb():
    """
    Test generation proteinDB from altORFs using dnaseq-to-proteindb tool
    :return:
    """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['cosmic-to-proteindb', '--config_file', 'config/cosmic_config.yaml',
                            '--input_mutation', 'testdata/test_cosmic_mutations.tsv',
                            '--input_genes', 'testdata/test_cosmic_genes.fa',
                            '--output_db', 'testdata/test_cosmic_mutations_proteindb.fa',
                            '--split_by_tissue_type', '--tissue_type', 'all'])
    assert result.exit_code == 0

def generate_decoy_database():
    """
        Test generation proteinDB from altORFs using dnaseq-to-proteindb tool
        :return:
        """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['generate-decoy', '--config_file', 'config/protein_decoy.yaml',
                            '--input', 'testdata/test_db.fa', '--output', 'testdata/output_decoy.fa'])
    assert result.exit_code == 0

def download_ensembl_data():
    """
        Test downloading ensembl data for species with taxonomy identifer = 9103
        :return:
        """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['ensembl-downloader', '--config_file', 'config/ensembl_downloader_config.yaml',
                            '--taxonomy', '9103', '--output_directory', 'testdata'])
    assert result.exit_code == 0

def download_cbioportal_data():
    """
        Test downloading cbioportal data for study id: paac_jhu_2014
        :return:
        """
    runner = CliRunner()
    result = runner.invoke(cli,
                           ['cbioportal-downloader', '--config_file', 'config/cbioportal_config.yaml',
                            '--download_study', 'paac_jhu_2014', '--output_directory', 'testdata'])
    assert result.exit_code == 0

if __name__ == '__main__':
    vcf_to_proteindb()
    vcf_gnomad_to_proteindb()
    dnaseq_to_proteindb()
    dnaseq_ncrnas_to_proteindb()
    dnaseq_lncrnas_to_proteindb()
    dnaseq_sncrnas_to_proteindb()
    dnaseq_pseudogenes_to_proteindb()
    dnaseq_altorfs_to_proteindb()
    cbioportal_to_proteindb()
    generate_decoy_database()
    cosmic_to_proteindb()
    download_ensembl_data()
    download_cbioportal_data()



from setuptools import setup, find_packages


def readme():
    with open('README.md') as f:
        return f.read()

setup(name='pypgatk',
      version='0.0.2',
      description='Python tools for proteogenomics ',
      url='http://github.com/bigbio/py-pgatk',
      long_description=readme(),
      long_description_content_type='text/markdown',
      author='PgAtk Team',
      author_email='ypriverol@gmail.com',
      license='Apache 2',
      install_requires=[
            'Click==7.0',
            'requests==2.21.0',
            'biopython==1.73'],
      scripts=['pypgatk/pypgatk_cli.py'],
      packages=find_packages(),
      zip_safe=False)



import logging

import click

from pypgatk.cgenomes.cosmic_downloader import CosmicDownloadService
from pypgatk.toolbox.exceptions import AppConfigException


@click.command('cosmic-downloader', short_help='Command to download the cosmic mutation database')
@click.option('--config_file',
              '-c',
              help='Configuration file for the ensembl data downloader pipeline',
              default='config/cosmic_config.yaml')
@click.option('--output_directory',
              '-o',
              help='Output directory for the peptide databases',
              default="./database_cosmic/")
@click.option('--username', '-u',
              help="Username for cosmic database -- please if you don't have one register here (https://cancer.sanger.ac.uk/cosmic/register)")
@click.option('--password', '-p',
              help="Password for cosmic database -- please if you don't have one register here (https://cancer.sanger.ac.uk/cosmic/register)")
@click.pass_context
def cosmic_downloader(ctx, config_file, output_directory, username, password):
    if config_file is None:
        msg = "The config file for the pipeline is missing, please provide one "
        logging.error(msg)
        raise AppConfigException(msg)

    pipeline_arguments = {}
    if output_directory is not None:
        pipeline_arguments[CosmicDownloadService.CONFIG_OUTPUT_DIRECTORY] = output_directory
    if username is not None:
        pipeline_arguments[CosmicDownloadService.CONFIG_COSMIC_FTP_USER] = username

    if password is not None:
        pipeline_arguments[CosmicDownloadService.CONFIG_COSMIC_FTP_PASSWORD] = password

    cosmic_downloader_service = CosmicDownloadService(config_file, pipeline_arguments)

    cosmic_downloader_service.download_mutation_file()





class SNP(object):

    def __init__(self, gene=None, mrna=None, dna_mut=None, aa_mut=None, type=None):
        """
        Default constructor of SNP
        :param gene:
        :param mRNA:
        :param dna_mut:
        :param aa_mut:
        :param type:
        """
        self.gene = gene
        self.mrna = mrna
        self.aa_mut = aa_mut
        self.type = type
        self.dna_mut = dna_mut


import logging

import click

from pypgatk.ensembl.data_downloader import EnsemblDataDownloadService
from pypgatk.toolbox.exceptions import AppConfigException


@click.command('ensembl-downloader', short_help='Command to download the ensembl information')
@click.option('--config_file',
              '-c',
              help='Configuration file for the ensembl data downloader pipeline',
              default='config/ensembl_downloader_config.yaml')
@click.option('--output_directory',
              '-o',
              help='Output directory for the peptide databases',
              default="./database_ensembl/")
@click.option('--folder_prefix_release',
              '-fp', help='Output folder prefix to download the data',
              default='release-')
@click.option('--taxonomy',
              '-t',
              help='Taxonomy identifiers (comma separated list can be given) that will be use to download the data from Ensembl',
              default='')
@click.option('--list_taxonomies', '-l',
              help='Print the list of all the taxonomies in ENSEMBL (https://www.ensembl.org)', is_flag=True, default=False)
@click.option('--skip_gtf', '-sg', help="Skip the gtf file during the download", is_flag=True)
@click.option('--skip_protein', '-sp', help="Skip the protein fasta file during download", is_flag=True)
@click.option('--skip_cds', '-sc', help='Skip the CDS file download', is_flag=True)
@click.option('--skip_cdna', '-sd', help='Skip the cDNA file download', is_flag=True)
@click.option('--skip_ncrna', '-sn', help='Skip the ncRNA file download', is_flag=True)
@click.option('--skip_vcf', '-sv', help='Skip the VCF variant file', is_flag=True)
def ensembl_downloader(config_file, output_directory, folder_prefix_release, taxonomy, list_taxonomies,
                       skip_gtf, skip_protein, skip_cds, skip_cdna, skip_ncrna, skip_vcf):
    """ This tool enables to download from enseml ftp the FASTA and GTF files"""

    if config_file is None:
        msg = "The config file for the pipeline is missing, please provide one "
        logging.error(msg)
        raise AppConfigException(msg)

    # Parse pipelines parameters.
    pipeline_arguments = {}
    if output_directory is not None:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_OUTPUT_DIRECTORY] = output_directory
    if folder_prefix_release is not None:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_FOLDER_PREFIX_RELEASE] = folder_prefix_release
    if taxonomy is not None:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_TAXONOMY] = taxonomy
    if list_taxonomies:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_LIST_TAXONOMIES] = list_taxonomies
    if skip_protein is not None and skip_protein:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_PROTEIN] = True
    else:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_PROTEIN] = False
    if skip_gtf is not None and skip_gtf:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_GTF] = True
    else:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_GTF] = False
    if skip_cds is not None and skip_cds:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_CDS] = True
    else:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_CDS] = False
    if skip_ncrna is not None and skip_ncrna:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_NCRNA] = True
    else:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_NCRNA] = False
    if skip_cdna is not None and skip_cdna:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_CDNA] = True
    else:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_CDNA] = False
    if skip_vcf is not None and skip_vcf:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_VCF] = True
    else:
        pipeline_arguments[EnsemblDataDownloadService.CONFIG_KEY_SKIP_VCF] = False

    ensembl_download_service = EnsemblDataDownloadService(config_file, pipeline_arguments)

    
    logger = ensembl_download_service.get_logger_for("Main Pipeline Ensembl Downloader")
    logger.info("Pipeline STARTING ... ")
    if list_taxonomies:
        list_of_taxonomies = ensembl_download_service.get_species_from_rest()
        for taxonomy_info in list_of_taxonomies: 
            print(taxonomy_info)
    
    ensembl_download_service.download_database_by_species()

    logger.info("Pipeline Finish !!!")